@@ -33,6 +33,7 @@ import json
 import time
 import logging
 from pyqtgraph.graphicsItems.ROI import ROI, LineROI, Handle
+from pyqtgraph.graphicsItems.IsocurveItem import IsocurveItem
 
 _VMAJOR, _VMINOR, _VPATCH = _pg.__version__.split(".")[:3] \
     if _pg.__version__ else ("0", "9", "0")
@@ -136,6 +137,41 @@ class SimpleLineROI(LineROI):
         return [pos1.x(), pos1.y(), pos2.x(), pos2.y(), size.y()]
 
 
+class RegionItem(IsocurveItem):
+
+    def __init__(self, points=None, pen='w', **args):
+        """ constructor
+
+        :param points: list of points
+        :type points:  :obj:`list` < (float, float) >
+        :param pen: qt pen
+        :type pen: :class:`pyqtgraph.QtGui.QPen`
+        :param args: more params
+        :type args: :obj:`dict` <:obj:`str`, `any`>
+        """
+        IsocurveItem.__init__(self, points, 0, pen, None, **args)
+        # if points and points[0] and points[0][0]:
+        #     self.setPos(*points[0][0])
+
+    def generatePath(self):
+        """ generate QPainterPath
+        """
+        if self.data is None:
+            self.path = None
+            return
+
+        self.path = _pg.QtGui.QPainterPath()
+        # nopos = True
+        for line in self.data:
+            if line and len(line) > 1:
+                self.path.moveTo(*line[0])
+                # if nopos:
+                #     self.setPos(*line[0])
+                #     nopos = False
+                for p in line[1:]:
+                    self.path.lineTo(*p)
+
+
 class DisplayExtension(QtCore.QObject):
     """ display extension for ImageDisplayWidget
     """
@@ -274,7 +310,7 @@ class ROIExtension(DisplayExtension):
         self.__roitext.append(text)
         self._mainwidget.viewbox().addItem(self.__roi[0])
         self.__roi[0].hide()
-        self.setROIsColors()
+        self.setColors()
 
         self.__roiregionmapper.mapped.connect(self.changeROIRegion)
         self.__currentroimapper.mapped.connect(self._emitROICoordsChanged)
@@ -331,7 +367,7 @@ class ROIExtension(DisplayExtension):
         self._mainwidget.viewbox().addItem(self.__roi[-1])
 
         self.__coords.append(coords)
-        self.setROIsColors()
+        self.setColors()
 
     def __removeROI(self):
         """ removes the last roi
@@ -542,8 +578,8 @@ class ROIExtension(DisplayExtension):
             self.__removeROI()
         self.__showROIs(self._enabled)
 
-    def setROIsColors(self, colors=None):
-        """ sets statistics without scaling flag
+    def setColors(self, colors=None):
+        """ sets colors
 
         :param colors: json list of roi colors
         :type colors: :obj:`str`
@@ -1107,6 +1143,251 @@ class MeshExtension(DisplayExtension):
             crd.setSize([size[1], size[0]])
 
 
+class RegionsExtension(DisplayExtension):
+
+    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) region points changed signal
+    regionPointsChanged = QtCore.pyqtSignal()
+
+    def __init__(self, parent=None):
+        """ constructor
+
+        :param parent: parent object
+        :type parent: :class:`pyqtgraph.QtCore.QObject`
+        """
+        DisplayExtension.__init__(self, parent)
+
+        #: (:obj:`str`) tool name
+        self.name = "regions"
+
+        #: (:obj:`int`) current roi id
+        self.__current = 0
+        #: (:obj:`list` < [int, int, int, int] > )
+        #: x1,y1,x2,y2 regions coordinates
+        self.__points = [[[(0, 0)]]]
+        #: (:obj:`list` < (int, int, int) > ) list with region colors
+        self.__colors = []
+
+        #: ( (:obj:`int`, :obj:`int', :obj:`int`)) default pen color
+        self.__defpen = (0, 255, 127)
+        #: (:obj:`list` <:class:`pyqtgraph.graphicsItems.TextItem`>)
+        #:            list of region widgets
+        self.__regiontext = []
+        #: (:obj:`list` <:class:`pyqtgraph.graphicsItems.ROI`>)
+        #:            list of region widgets
+        self.__region = []
+        self.__region.append(RegionItem(self.__points[0],
+                                        pen=_pg.mkPen('#00ff7f', width=2)))
+        # text = _pg.TextItem("1.", anchor=(1, 1))
+        # text.setParentItem(self.__region[0])
+        # self.__regiontext.append(text)
+        # clr = '#00ff7f'
+        # if hasattr(self.__regiontext[0], "setColor"):
+        #     self.__regiontext[0].setColor(clr)
+        # else:
+        #     self.__regiontext[0].color = _pg.functions.mkColor(clr)
+        #     self.__regiontext[0].textItem.setDefaultTextColor(
+        #         self.__regiontext[it].color)
+        self._mainwidget.viewbox().addItem(self.__region[0])
+        self.__region[0].hide()
+        self.setColors()
+
+    def show(self, parameters):
+        """ set subwidget properties
+
+        :param parameters: tool parameters
+        :type parameters: :class:`lavuelib.toolWidget.ToolParameters`
+        """
+        if parameters.regions is not None:
+            self.__showRegions(parameters.regions)
+            self._enabled = parameters.regions
+
+    def scalingLabel(self):
+        """ provides scaling label
+
+        :returns:  scaling label
+        :rtype: str
+        """
+        return "intensity"
+
+    def __addRegion(self, points=None, rid=0):
+        """ adds Regions
+
+        :param points: region coordinates
+        :type points: :obj:`list`
+                 < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        """
+        if not points or not isinstance(points, list):
+            points = [[(0, 0)]]
+
+        if self._mainwidget.transformations()[0]:
+            points = [[(p[1], p[0]) for p in pt] for pt in points]
+        clr = tuple(self.__colors[rid % len(self.__colors)]) \
+            if self.__colors else self.__defpen
+        self.__region.append(
+            RegionItem(points, pen=_pg.mkPen(clr, width=2)))
+        # text = _pg.TextItem("%s." % len(self.__region), anchor=(1, 1))
+        # text.setParentItem(self.__region[-1])
+        # self.__regiontext.append(text)
+        self._mainwidget.viewbox().addItem(self.__region[-1])
+
+        self.__points.append(points)
+        # self.setColors()
+
+    def __removeRegion(self):
+        """ removes the last region
+        """
+        region = self.__region.pop()
+        region.hide()
+        # regiontext = self.__regiontext.pop()
+        # regiontext.hide()
+        self._mainwidget.viewbox().removeItem(region)
+        self.__points.pop()
+
+    def _getRegion(self, rid=-1):
+        """ get the given or the last region
+
+        :param rid: region id
+        :type rid: :obj:`int`
+        """
+        if self.__region and len(self.__region) > rid:
+            return self.__region[rid]
+        else:
+            return None
+
+    def __showRegions(self, status):
+        """ shows or hides regions
+
+        :param status: will be shown
+        :type status: :obj:`bool`
+        """
+        if status:
+            for rng in self.__region:
+                rng.show()
+        else:
+            for rng in self.__region:
+                rng.hide()
+
+    def __addRegionPoints(self, points):
+        """ adds region coorinates
+
+        :param points: region coordinates
+        :type points: :obj:`list`
+                < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        """
+        if points:
+            for i, crd in enumerate(self.__region):
+                if i < len(points):
+                    self.__points[i] = points[i]
+                    if self._mainwidget.transformations()[0]:
+                        pnts = [[(p[1], p[0]) for p in pt] for pt in points[i]]
+                    else:
+                        pnts = points[i]
+                    crd.setData(pnts)
+
+    def updateRegions(self, points, rid=None):
+        """ update Regions
+
+        :param rid: rng id
+        :type rid: :obj:`int`
+        :param points: rng coordinates
+        :type points: :obj:`list` <  :obj:`list` <  :obj:`list`
+                  < (:obj:`float`, :obj:`float`) > > >
+        """
+        if rid is None:
+            if points is None:
+                points = []
+            rid = len(points)
+        self.__addRegionPoints(points)
+        while rid > len(self.__region):
+            if points and len(points) >= len(self.__region):
+                self.__addRegion(points[len(self.__region)],
+                                 rid=len(self.__region))
+            else:
+                self.__addRegion(rid=len(self.__region))
+        if rid <= 0:
+            self.__current = -1
+        elif self.__current >= rid:
+            self.__current = 0
+        while self._getRegion(max(rid, 0)) is not None:
+            self.__removeRegion()
+        self.__showRegions(self._enabled)
+
+    def setColors(self, colors=None):
+        """ sets colors
+
+        :param colors: json list of roi colors
+        :type colors: :obj:`str`
+        :returns: change status
+        :rtype: :obj:`bool`
+        """
+        force = False
+        if colors is not None:
+            colors = json.loads(colors)
+            if not isinstance(colors, list):
+                return False
+            for cl in colors:
+                if not isinstance(cl, list):
+                    return False
+                if len(cl) != 3:
+                    return False
+                for clit in cl:
+                    if not isinstance(clit, int):
+                        return False
+        else:
+            colors = self.__colors
+            force = True
+        if self.__colors != colors or force:
+            self.__colors = colors
+            for it, rg in enumerate(self.__region):
+                clr = tuple(colors[it % len(colors)]) if colors \
+                    else self.__defpen
+                rg.setPen(_pg.mkPen(clr, width=2))
+                # if hasattr(self.__regiontext[it], "setColor"):
+                #     self.__regiontext[it].setColor(clr)
+                # else:
+                #     self.__regiontext[it].color = _pg.functions.mkColor(clr)
+                #     self.__regiontext[it].textItem.setDefaultTextColor(
+                #         self.__regiontext[it].color)
+        return True
+
+    def regionPoints(self):
+        """ provides region coordinates
+
+        :return: region coordinates
+        :rtype: :obj:`list` <  :obj:`list` <  :obj:`list`
+                  < (:obj:`float`, :obj:`float`) > > >
+        """
+        return self.__points
+
+    def isRegionEnabled(self):
+        """ provides flag regions enabled
+
+        :return: region enabled flag
+        :rtype: :obj:`bool`
+        """
+        return self._enabled
+
+    def currentRegion(self):
+        """ provides current region id
+
+        :return: region id
+        :rtype: :obj:`int`
+        """
+        return self.__current
+
+    def transpose(self):
+        """ transposes Regions
+        """
+        for i, crd in enumerate(self.__region):
+            if i < len(self.__points):
+                if self._mainwidget.transformations()[0]:
+                    pnts = [[(p[1], p[0]) for p in pt]
+                            for pt in self.__points[i]]
+                else:
+                    pnts = self.__points[i]
+                crd.setData(pnts)
+
+
 class LockerExtension(DisplayExtension):
 
     def __init__(self, parent=None):
@@ -1569,17 +1850,19 @@ class MarkExtension(DisplayExtension):
             self.updatePositionMark(x, y)
 
     @QtCore.pyqtSlot(float, float)
-    def updatePositionMark(self, xdata, ydata):
+    def updatePositionMark(self, xdata, ydata, scaled=False):
         """ updates the position mark
 
         :param xdata: x pixel position
         :type xdata: :obj:`float`
         :param ydata: y-pixel position
         :type ydata: :obj:`float`
+        :param scaled: scaled flag
+        :type scaled: :obj:`bool`
         """
         self.__markcoordinates = [xdata, ydata]
         pos0, pos1, scale0, scale1 = self._mainwidget.scale()
-        if pos0 is not None:
+        if pos0 is not None and not scaled:
             if not self._mainwidget.transformations()[0]:
                 self.__markVLine.setPos((xdata) * scale0 + pos0)
                 self.__markHLine.setPos((ydata) * scale1 + pos1)
