@@ -32,7 +32,7 @@ import threading
 import numpy
 
 
-#: (:mod:`PNIWriter` or :mod:`H5PYWriter`or :mod:`H5CppWriter`)
+#: (:mod:`H5PYWriter`or :mod:`H5CppWriter`)
 #    default writer module
 writer = None
 
@@ -86,6 +86,31 @@ def create_file(filename, overwrite=False, **pars):
     return fl
 
 
+def load_file(membuffer, filename=None, readonly=False, **pars):
+    """ load a file from memory byte buffer
+
+    :param membuffer: memory buffer
+    :type membuffer: :obj:`bytes` or :obj:`io.BytesIO`
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param readonly: readonly flag
+    :type readonly: :obj:`bool`
+    :param pars: parameters
+    :type pars: :obj:`dict` < :obj:`str`, :obj:`str`>
+    :returns: file objects
+    :rtype: :class:`H5PYFile` or :class:`H5CppFile`
+    """
+    if 'writer' in pars.keys():
+        wr = pars.pop('writer')
+    else:
+        with writerlock:
+            wr = writer
+    fl = wr.load_file(membuffer, filename, readonly, **pars)
+    if hasattr(fl, "writer"):
+        fl.writer = wr
+    return fl
+
+
 def link(target, parent, name):
     """ create link
 
@@ -168,17 +193,169 @@ def data_filter(parent=None):
 deflate_filter = data_filter
 
 
+def unlimited(parent=None):
+    """ return dataspace UNLIMITED variable for the current writer module
+
+    :param parent: parent object
+    :type parent: :class:`FTObject`
+    :returns:  dataspace UNLIMITED variable
+    :rtype: :class:`h5cpp.dataspace.UNLIMITED` or :class:`h5py.UNLIMITED`
+    """
+    node = parent
+    wr = None
+    while node:
+        if hasattr(node, "writer"):
+            wr = node.writer
+            break
+        else:
+            if hasattr(node, "parent"):
+                node = node.parent
+            else:
+                break
+    if not wr:
+        with writerlock:
+            wr = writer
+    return wr.unlimited()
+
+
+def target_field_view(filename, fieldpath, shape,
+                      dtype=None, maxshape=None, parent=None):
+    """ create target field view for VDS
+
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param fieldpath: nexus field path
+    :type fieldpath: :obj:`str`
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :param parent: parent object
+    :type parent: :class:`FTObject`
+    :returns: target field object
+    :rtype: :class:`FTTargetFieldView`
+    """
+    node = parent
+    wr = None
+    while node:
+        if hasattr(node, "writer"):
+            wr = node.writer
+            break
+        else:
+            if hasattr(node, "parent"):
+                node = node.parent
+            else:
+                break
+    if not wr:
+        with writerlock:
+            wr = writer
+    return wr.target_field_view(filename, fieldpath, shape,
+                                dtype, maxshape)
+
+
+def virtual_field_layout(shape, dtype, maxshape=None, parent=None):
+    """ creates a virtual field layout for a VDS file
+
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :param parent: parent object
+    :type parent: :class:`FTObject`
+    :returns: virtual layout
+    :rtype: :class:`FTVirtualFieldLayout`
+    """
+    node = parent
+    wr = None
+    while node:
+        if hasattr(node, "writer"):
+            wr = node.writer
+            break
+        else:
+            if hasattr(node, "parent"):
+                node = node.parent
+            else:
+                break
+    if not wr:
+        with writerlock:
+            wr = writer
+    return wr.virtual_field_layout(shape, dtype, maxshape)
+
+
 def setwriter(wr):
     """ sets writer
 
     :param wr: writer module
-    :type wr: :mod:`PNIWriter` or :mod:`H5PYWriter` or :mod:`H5CppWriter`
+    :type wr: :mod:`H5PYWriter` or :mod:`H5CppWriter`
     """
     global writer
     with writerlock:
         writer = wr
 
 
+class FTHyperslab(object):
+    """ hyperslab class """
+
+    def __init__(self, offset=None, block=None, count=None, stride=None):
+        """ constructor
+
+        :param offset: a list of offsets
+        :type offset: :obj:`tuple`
+        :param block: a list of blocks
+        :type block: :obj:`tuple`
+        :param count: a list of counts
+        :type count: :obj:`tuple`
+        :param stride: a list of strides
+        :type stride: :obj:`tuple`
+        """
+        self.offset = offset
+        self.block = block
+        self.count = count
+        self.stride = stride
+
+    def __eq__(self, other):
+        """ compares hyporslabs
+
+        :param other: hyperslab to compare
+        :type other: :class:`FTHyperslab`
+        :returns: if they are equal
+        :rtype: :obj:`bool`
+        """
+        return hasattr(other, 'offset') and \
+            hasattr(other, 'block') and \
+            hasattr(other, 'count') and \
+            hasattr(other, 'stride') and \
+            self.offset == other.offset and \
+            self.block == other.block and \
+            self.count == other.count and \
+            self.stride == other.stride
+
+    def __ne__(self, other):
+        """ compares hyporslabs
+
+        :param other: hyperslab to compare
+        :type other: :class:`FTHyperslab`
+        :returns: if they are not equal
+        :rtype: :obj:`bool`
+        """
+        return not self.__eq__(other)
+
+    def __len__(self):
+        """ dimension length of hyperslab
+
+        :returns: dimension length of hyperslab
+        :rtype: :obj:`int`
+        """
+        return max(len(self.offset or []),
+                   len(self.block or []),
+                   len(self.count or []),
+                   len(self.stride or []))
+
+
 class FTObject(object):
 
     """ virtual file tree object
@@ -249,7 +426,7 @@ class FTObject(object):
     def h5object(self):
         """ provide object of native library
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
         return self._h5object
@@ -272,10 +449,12 @@ def first(array):
     :returns: first element of the array
     :type array: :obj:`any`
     """
-    if isinstance(array, numpy.ndarray) and len(array) == 1:
-        return array[0]
-    else:
-        return array
+    try:
+        if isinstance(array, numpy.ndarray) and len(array) == 1:
+            return array[0]
+    except Exception:
+        pass
+    return array
 
 
 class FTFile(FTObject):
@@ -286,18 +465,17 @@ class FTFile(FTObject):
     def __init__(self, h5object, filename):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param filename:  file name
         :type filename: :obj:`str`
         :param writer: writer module
-        :type writer: :mod:`PNIWriter` or :mod:`H5PYWriter`
-                        or :mod:`H5CppWriter`
+        :type writer: :mod:`H5PYWriter` or :mod:`H5CppWriter`
         """
         FTObject.__init__(self, h5object, None)
         #: (:obj:`str`) file name
         self.name = filename
-        #: (:mod:`PNIWriter` or :mod:`H5PYWriter` or :mod:`H5CppWriter`)
+        #: (:mod:`H5PYWriter` or :mod:`H5CppWriter`)
         # writer module
         self.writer = None
 
@@ -353,6 +531,76 @@ class FTFile(FTObject):
         starttime = tz.localize(datetime.datetime.now())
         return str(starttime.strftime(fmt))
 
+    def default_field(self):
+        """ field pointed by default attributes
+
+        :returns: field pointed by default attributes
+        :rtype: :class:`FTField`
+        """
+        node = self.root()
+        searching = True
+        while searching:
+            attrs = node.attributes
+            if hasattr(node, "names") and "default" in attrs.names():
+                nname = attrs["default"].read()
+                if isinstance(nname, numpy.ndarray) and len(nname):
+                    nname = nname[0]
+                if nname in node.names():
+                    node = node.open(nname)
+                    continue
+            searching = False
+        if hasattr(node, "names"):
+            attrs = node.attributes
+            if "signal" in attrs.names():
+                nname = attrs["signal"].read()
+                if isinstance(nname, numpy.ndarray) and len(nname):
+                    nname = nname[0]
+                if nname in node.names():
+                    node = node.open(nname)
+        if not hasattr(node, "names"):
+            return node
+
+        for cnm in ["NXentry", "NXdata", "NXmonitor", "NXlog"]:
+            names = node.names()
+            if cnm[2:] in names:
+                snames = [cnm[2:]]
+            else:
+                snames = []
+            snames.extend(sorted([nm for nm in names if nm != cnm[2:]]))
+            for nn in snames:
+                nd = node.open(nn)
+                if not hasattr(nd, "attributes"):
+                    continue
+                attrs = nd.attributes
+                if "NX_class" in attrs.names():
+                    nname = attrs["NX_class"].read()
+                    if isinstance(nname, numpy.ndarray) and len(nname):
+                        nname = nname[0]
+                    if nname in cnm:
+                        node = nd
+                        break
+        if hasattr(node, "names") and hasattr(node, "attributes"):
+            attrs = node.attributes
+            if "signal" in attrs.names():
+                nname = attrs["signal"].read()
+                if isinstance(nname, numpy.ndarray) and len(nname):
+                    nname = nname[0]
+                if nname in node.names():
+                    node = node.open(nname)
+        if not hasattr(node, "names"):
+            return node
+        while hasattr(node, "names") and "data" in node.names():
+            node = node.open("data")
+        if hasattr(node, "names"):
+            for nn in sorted(node.names()):
+                nd = node.open(nn)
+                if not hasattr(nd, "names"):
+                    node = nd
+                    break
+        if not hasattr(node, "names"):
+            return node
+        return None
+
 
 class FTGroup(FTObject):
 
@@ -362,7 +610,7 @@ class FTGroup(FTObject):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -402,6 +650,17 @@ class FTGroup(FTObject):
         :rtype: :class:`FTGroup`
         """
 
+    def create_virtual_field(self, name, layout, fillvalue=None):
+        """ creates a virtual filed tres element
+
+        :param name: group name
+        :type name: :obj:`str`
+        :param layout: virual field layout
+        :type layout: :class:`H5CppFieldLayout`
+        :param fillvalue:  fill value
+        :type fillvalue: :obj:`int` or :class:`np.ndarray`
+        """
+
     def create_field(self, name, type_code,
                      shape=None, chunk=None, dfilter=None):
         """ open a file tree element
@@ -448,7 +707,7 @@ class FTGroup(FTObject):
     def names(self):
         """ read the child names
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`list` <`str`>
         """
 
@@ -458,15 +717,63 @@ class FTGroup(FTObject):
         FTObject._reopen(self)
 
 
+class FTVirtualFieldLayout(FTObject):
+
+    """ virtual field layout """
+
+    def __init__(self, h5object=None):
+        """ constructor
+
+        :param h5object: h5 object
+        :type h5object: :obj:`any`
+        """
+        FTObject.__init__(self, h5object)
+
+    def __setitem__(self, key, source):
+        """ add target field view to layout
+
+        :param key: slide or selection
+        :type key: :obj:`tuple`
+        :param source: target field view
+        :type source: :class:`FTTargetFieldView`
+        """
+        self._h5object.__setitem__(key, source._h5object)
+
+    def add(self, key, source, sourcekey=None):
+        """ add target field to layout
+
+        :param key: slide or selection
+        :type key: :obj:`tuple`
+        :param source: target field
+        :type source: :class:`FTTargetFieldView`
+        :param sourcekey: slide or selection
+        :type sourcekey: :obj:`tuple`
+        """
+        self._h5object.add(key, source, sourcekey)
+
+
+class FTTargetFieldView(FTObject):
+
+    """ target field view for VDS"""
+
+    def __init__(self, h5object=None):
+        """ constructor
+
+        :param h5object: h5 object
+        :type h5object: :obj:`any`
+        """
+        FTObject.__init__(self, h5object)
+
+
 class FTField(FTObject):
 
-    """ file tree file
+    """ file writer field
     """
 
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -500,14 +807,14 @@ class FTField(FTObject):
     def read(self):
         """ read the field value
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
 
     def write(self, o):
         """ write the field value
 
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
 
@@ -516,7 +823,7 @@ class FTField(FTObject):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
 
@@ -525,7 +832,7 @@ class FTField(FTObject):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
 
@@ -567,7 +874,7 @@ class FTLink(FTObject):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -603,7 +910,7 @@ class FTDataFilter(FTObject):
     def __init__(self, h5object=None, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -708,7 +1015,7 @@ class FTAttributeManager(FTObject):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -746,6 +1053,13 @@ class FTAttributeManager(FTObject):
         :rtype: :class:`FTAtribute`
         """
 
+    def names(self):
+        """ key values
+
+        :returns: attribute names
+        :rtype: :obj:`list` <:obj:`str`>
+        """
+
     def reopen(self):
         """ reopen attribute
         """
@@ -760,7 +1074,7 @@ class FTAttribute(FTObject):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
