@@ -22,7 +22,8 @@
 #     Jan Kotanski <jan.kotanski@desy.de>
 #
 
-""" Provides pni file writer """
+
+""" Provides h5cpp file writer """
 
 import math
 import os
@@ -73,6 +74,65 @@ def _tostr(text):
         return str(text)
 
 
+def unlimited_selection(sel, shape):
+    """ checks if hyperslab is unlimited
+
+    :param sel: hyperslab selection
+    :type sel: :class:`filewriter.FTHyperslab`
+    :param shape: give shape
+    :type shape: :obj:`list`
+    :returns: if hyperslab is unlimited list
+    :rtype: :obj:`list` <:obj:`bool`>
+    """
+    res = None
+    if isinstance(sel, tuple):
+        res = []
+        for sl in sel:
+            if hasattr(sl, "stop"):
+                res.append(
+                    True if sl.stop in [unlimited()]
+                    else False)
+            elif hasattr(sl, "count"):
+                res.append(
+                    True if sl.count in [unlimited()]
+                    else False)
+            else:
+                res.append(
+                    True if sl in [unlimited()]
+                    else False)
+
+    elif hasattr(sel, "count"):
+        res = []
+        for ct in sel.count():
+            res.append(
+                True if ct in [unlimited()]
+                else False)
+    elif isinstance(sel, slice):
+        res = [True if sel.stop in [unlimited()]
+               else False]
+
+    elif sel in [unlimited()]:
+        res = [True]
+    lsh = len(shape)
+    lct = len(res)
+    ln = max(lsh, lct)
+    if res and any(t is True for t in res):
+        offset = [0 for _ in range(ln)]
+        block = [1 for _ in range(ln)]
+        stride = [1 for _ in range(ln)]
+        count = list(shape)
+        while lct > len(count):
+            count.append(1)
+        for si in range(lct):
+            if res[si]:
+                count[si] = h5cpp.dataspace.UNLIMITED
+        # print("Hyperslab %s %s %s %s" % (offset, block, count, stride))
+        return h5cpp.dataspace.Hyperslab(
+            offset=offset, block=block, count=count, stride=stride)
+    else:
+        return None
+
+
 def _slice2selection(t, shape):
     """ converts slice(s) to selection
 
@@ -85,6 +145,36 @@ def _slice2selection(t, shape):
     """
     if t is Ellipsis:
         return None
+    elif isinstance(t, filewriter.FTHyperslab):
+        offset = list(t.offset or [])
+        block = list(t.block or [])
+        count = list(t.count or [])
+        stride = list(t.stride or [])
+        for dm, sz in enumerate(shape):
+            if len(offset) > dm:
+                if offset[dm] is None:
+                    offset[dm] = 0
+            else:
+                offset.append(0)
+            if len(block) > dm:
+                if block[dm] is None:
+                    block[dm] = 1
+            else:
+                block.append(1)
+            if len(count) > dm:
+                if count[dm] is None:
+                    count[dm] = sz
+            else:
+                count.append(sz)
+            if len(stride) > dm:
+                if stride[dm] is None:
+                    stride[dm] = 1
+            else:
+                stride.append(1)
+        # print("Hyperslab %s %s %s %s" % (offset, block, count, stride))
+        return h5cpp.dataspace.Hyperslab(
+            offset=offset, block=block, count=count, stride=stride)
+
     elif isinstance(t, slice):
         start = t.start or 0
         stop = t.stop or shape[0]
@@ -99,7 +189,7 @@ def _slice2selection(t, shape):
             return h5cpp.dataspace.Hyperslab(
                 offset=(start,),
                 count=int(math.ceil((stop - start) / float(t.step))),
-                stride=(t.step - 1,))
+                stride=(t.step,))
     elif isinstance(t, (int, long)):
         return h5cpp.dataspace.Hyperslab(
             offset=(t,), block=(1,))
@@ -137,7 +227,7 @@ def _slice2selection(t, shape):
                     count.append(
                         int(math.ceil(
                             (stop - start) / float(tel.step))))
-                    stride.append(tel.step - 1)
+                    stride.append(tel.step)
             elif tel is Ellipsis:
                 esize = len(shape) - len(t) + 1
                 for jt in range(esize):
@@ -147,6 +237,7 @@ def _slice2selection(t, shape):
                     stride.append(1)
                     if jt < esize - 1:
                         it += 1
+        # print("Hyperslab %s %s %s %s" % (offset, block, count, stride))
         if len(offset):
             return h5cpp.dataspace.Hyperslab(
                 offset=offset, block=block, count=count, stride=stride)
@@ -193,6 +284,17 @@ hTp = {
 }
 
 
+def unlimited(parent=None):
+    """ return dataspace UNLIMITED variable for the current writer module
+
+    :param parent: parent object
+    :type parent: :class:`FTObject`
+    :returns:  dataspace UNLIMITED variable
+    :rtype: :class:`h5cpp.dataspace.UNLIMITED`
+    """
+    return h5cpp.dataspace.UNLIMITED
+
+
 def open_file(filename, readonly=False, libver=None, swmr=False):
     """ open the new file
 
@@ -228,6 +330,70 @@ def open_file(filename, readonly=False, libver=None, swmr=False):
     return H5CppFile(h5cpp.file.open(filename, flag, fapl), filename)
 
 
+def is_image_file_supported():
+    """ provides if loading of image files are supported
+
+    :retruns: if loading of image files are supported
+    :rtype: :obj:`bool`
+    """
+    return hasattr(h5cpp.file, "from_buffer") and \
+        hasattr(h5cpp.file, "ImageFlags")
+
+
+def is_vds_supported():
+    """ provides if vds are supported
+
+    :retruns: if vds are supported
+    :rtype: :obj:`bool`
+    """
+    return hasattr(h5cpp.property, "VirtualDataMaps") and \
+        hasattr(h5cpp.property, "VirtualDataMaps")
+
+
+def is_unlimited_vds_supported():
+    """ provides if unlimited vds are supported
+
+    :retruns: if unlimited vds are supported
+    :rtype: :obj:`bool`
+    """
+    return is_vds_supported()
+
+
+def load_file(membuffer, filename=None, readonly=False, **pars):
+    """ load a file from memory byte buffer
+
+    :param membuffer: memory buffer
+    :type membuffer: :obj:`bytes` or :obj:`io.BytesIO`
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param readonly: readonly flag
+    :type readonly: :obj:`bool`
+    :param pars: parameters
+    :type pars: :obj:`dict` < :obj:`str`, :obj:`str`>
+    :returns: file object
+    :rtype: :class:`H5PYFile`
+    """
+    if not is_image_file_supported():
+        raise Exception(
+            "Loading a file from a memory buffer not supported")
+    if type(membuffer).__name__ == "ndarray":
+        npdata = np.array(membuffer[:], dtype="uint8")
+    else:
+        if hasattr(membuffer, "getbuffer"):
+            membuffer = membuffer.getbuffer()
+        elif hasattr(membuffer, "getvalue"):
+            membuffer = membuffer.getvalue()
+        try:
+            npdata = np.frombuffer(membuffer[:], dtype=np.uint8)
+        except Exception:
+            npdata = np.fromstring(membuffer[:], dtype=np.uint8)
+    if readonly:
+        flag = h5cpp.file.ImageFlags.READONLY
+    else:
+        flag = h5cpp.file.ImageFlags.READWRITE
+    return H5CppFile(h5cpp.file.from_buffer(npdata, flag), filename)
+
+
 def create_file(filename, overwrite=False, libver=None, swmr=None):
     """ create a new file
 
@@ -255,9 +421,12 @@ def create_file(filename, overwrite=False, libver=None, swmr=None):
     attrs = rt.attributes
     attrs.create("file_time", pTh["unicode"]).write(
         unicode(H5CppFile.currenttime()))
-    attrs.create("HDF5_version", pTh["unicode"]).write(u"")
+    hdf5ver = u""
+    if hasattr(h5cpp, "current_library_version"):
+        hdf5ver = h5cpp.current_library_version()
+    attrs.create("HDF5_Version", pTh["unicode"]).write(hdf5ver)
     attrs.create("NX_class", pTh["unicode"]).write(u"NXroot")
-    attrs.create("NeXus_version", pTh["unicode"]).write(u"4.3.0")
+    # attrs.create("NeXus_version", pTh["unicode"]).write(u"4.3.0")
     attrs.create("file_name", pTh["unicode"]).write(unicode(filename))
     attrs.create("file_update_time", pTh["unicode"]).write(
         unicode(H5CppFile.currenttime()))
@@ -326,6 +495,46 @@ def data_filter():
 deflate_filter = data_filter
 
 
+def target_field_view(filename, fieldpath, shape,
+                      dtype=None, maxshape=None):
+    """ create target field view for VDS
+
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param fieldpath: nexus field path
+    :type fieldpath: :obj:`str`
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :returns: target field view object
+    :rtype: :class:`H5CppTargetFieldView`
+    """
+    return H5CppTargetFieldView(
+        filename, fieldpath, shape, dtype, maxshape)
+
+
+def virtual_field_layout(shape, dtype, maxshape=None):
+    """ creates a virtual field layout for a VDS file
+
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :returns: virtual layout
+    :rtype: :class:`H5CppVirtualFieldLayout`
+    """
+    if not is_vds_supported():
+        raise Exception("VDS not supported")
+    return H5CppVirtualFieldLayout(
+        h5cpp.property.VirtualDataMaps(),
+        shape, dtype, maxshape)
+
+
 class H5CppFile(filewriter.FTFile):
 
     """ file tree file
@@ -334,7 +543,7 @@ class H5CppFile(filewriter.FTFile):
     def __init__(self, h5object, filename):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param filename:  file name
         :type filename: :obj:`str`
@@ -439,7 +648,7 @@ class H5CppGroup(filewriter.FTGroup):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: tree parent
         :type tparent: :obj:`FTObject`
@@ -533,6 +742,40 @@ class H5CppGroup(filewriter.FTGroup):
                 "NX_class", pTh["unicode"]).write(unicode(nxclass))
         return H5CppGroup(gr, self)
 
+    def create_virtual_field(self, name, layout, fillvalue=0):
+        """ creates a virtual filed tres element
+
+        :param name: field name
+        :type name: :obj:`str`
+        :param layout: virual field layout
+        :type layout: :class:`H5CppFieldLayout`
+        :param fillvalue:  fill value
+        :type fillvalue: :obj:`int` or :class:`np.ndarray`
+        """
+        if not is_vds_supported():
+            raise Exception("VDS not supported")
+        dcpl = h5cpp.property.DatasetCreationList()
+        if fillvalue is not None:
+            if hasattr(dcpl, "set_fill_value"):
+                if isinstance(fillvalue, np.ndarray):
+                    dcpl.set_fill_value(
+                        fillvalue[0], pTh[str(fillvalue.dtype)])
+                else:
+                    dcpl.set_fill_value(
+                        fillvalue, pTh[_tostr(layout.dtype)])
+            else:
+                raise Exception("VDS fill_value not supported")
+
+        shape = layout.shape or [1]
+        dataspace = h5cpp.dataspace.Simple(
+            tuple(shape),
+            tuple([h5cpp.dataspace.UNLIMITED] * len(shape)))
+        vf = h5cpp.node.VirtualDataset(
+            self._h5object, h5cpp.Path(name),
+            pTh[_tostr(layout.dtype)], dataspace,
+            layout._h5object, dcpl=dcpl)
+        return H5CppField(vf, self)
+
     def create_field(self, name, type_code,
                      shape=None, chunk=None, dfilter=None):
         """ open a file tree element
@@ -551,33 +794,36 @@ class H5CppGroup(filewriter.FTGroup):
         :rtype: :class:`H5CppField`
         """
         dcpl = h5cpp.property.DatasetCreationList()
-        shape = shape or [1]
-        dataspace = h5cpp.dataspace.Simple(
-            tuple(shape), tuple([h5cpp.dataspace.UNLIMITED] * len(shape)))
-        if dfilter:
-            if dfilter.filterid == 1:
-                h5object = dfilter.h5object
-                h5object.level = dfilter.rate
-            else:
-                h5object = h5cpp.filter.ExternalFilter(
-                    dfilter.filterid, list(dfilter.options))
-            h5object(dcpl)
-            if dfilter.shuffle:
-                sfilter = h5cpp.filter.Shuffle()
-                sfilter(dcpl)
-        if chunk is None and shape is not None:
-            chunk = [(dm if dm != 0 else 1) for dm in shape]
-        dcpl.layout = h5cpp.property.DatasetLayout.CHUNKED
-        dcpl.chunk = tuple(chunk)
-        field = h5cpp.node.Dataset(
-            self._h5object, h5cpp.Path(name),
-            pTh[_tostr(type_code)], dataspace,
-            dcpl=dcpl)
-
-        fld = H5CppField(field, self)
-        # if type_code == "bool":
-        #     fld.boolflag = True
-        return fld
+        if type_code in ["str", "unicode", "string"] and \
+           shape is None and chunk is None:
+            dataspace = h5cpp.dataspace.Scalar()
+            return H5CppField(h5cpp.node.Dataset(
+                self._h5object, h5cpp.Path(name),
+                pTh[_tostr(type_code)], dataspace,
+                dcpl=dcpl), self)
+        else:
+            shape = shape or [1]
+            dataspace = h5cpp.dataspace.Simple(
+                tuple(shape), tuple([h5cpp.dataspace.UNLIMITED] * len(shape)))
+            if dfilter:
+                if dfilter.filterid == 1:
+                    h5object = dfilter.h5object
+                    h5object.level = dfilter.rate
+                else:
+                    h5object = h5cpp.filter.ExternalFilter(
+                        dfilter.filterid, list(dfilter.options))
+                h5object(dcpl)
+                if dfilter.shuffle:
+                    sfilter = h5cpp.filter.Shuffle()
+                    sfilter(dcpl)
+            if chunk is None and shape is not None:
+                chunk = [(dm if dm != 0 else 1) for dm in shape]
+            dcpl.layout = h5cpp.property.DatasetLayout.CHUNKED
+            dcpl.chunk = tuple(chunk)
+            return H5CppField(h5cpp.node.Dataset(
+                self._h5object, h5cpp.Path(name),
+                pTh[_tostr(type_code)], dataspace,
+                dcpl=dcpl), self)
 
     @property
     def size(self):
@@ -635,7 +881,7 @@ class H5CppGroup(filewriter.FTGroup):
     def names(self):
         """ read the child names
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`list` <`str`>
         """
         return [
@@ -700,7 +946,7 @@ class H5CppField(filewriter.FTField):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -765,12 +1011,13 @@ class H5CppField(filewriter.FTField):
         :param dim: size of the grow
         :type dim: :obj:`int`
         """
-        self._h5object.extent(dim, ext)
+        if self._h5object.dataspace.type != h5cpp.dataspace.Type.SCALAR:
+            self._h5object.extent(dim, ext)
 
     def read(self):
         """ read the field value
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
         if self.dtype in ['string', b'string']:
@@ -794,7 +1041,7 @@ class H5CppField(filewriter.FTField):
     def write(self, o):
         """ write the field value
 
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
         self._h5object.write(o)
@@ -804,7 +1051,7 @@ class H5CppField(filewriter.FTField):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
         if self.shape == (1,) and t == 0:
@@ -820,7 +1067,7 @@ class H5CppField(filewriter.FTField):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
         if self.shape == (1,) and t == 0:
@@ -966,6 +1213,8 @@ class H5CppField(filewriter.FTField):
         """
         if hasattr(self._h5object.dataspace, "current_dimensions"):
             return self._h5object.dataspace.current_dimensions
+        if self._h5object.dataspace.type == h5cpp.dataspace.Type.SCALAR:
+            return ()
         else:
             return (1,)
 
@@ -987,7 +1236,7 @@ class H5CppLink(filewriter.FTLink):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -1027,6 +1276,13 @@ class H5CppLink(filewriter.FTLink):
 
     @classmethod
     def getfilename(cls, obj):
+        """ provides a filename from h5 node
+
+        :param obj: h5 node
+        :type obj: :class:`FTObject`
+        :returns: file name
+        :rtype: :obj:`str`
+        """
         filename = ""
         while not filename:
             par = obj.parent
@@ -1077,8 +1333,117 @@ class H5CppDataFilter(filewriter.FTDataFilter):
     """
 
 
+class H5CppVirtualFieldLayout(filewriter.FTVirtualFieldLayout):
+
+    """ virtual field layout """
+
+    def __init__(self, h5object, shape, dtype=None, maxshape=None):
+        """ constructor
+
+        :param h5object: h5 object
+        :type h5object: :obj:`any`
+        :param shape: shape
+        :type shape: :obj:`list` < :obj:`int` >
+        :param dtype: attribute type
+        :type dtype: :obj:`str`
+        :param maxshape: shape
+        :type maxshape: :obj:`list` < :obj:`int` >
+        """
+        filewriter.FTVirtualFieldLayout.__init__(self, h5object)
+        #: (:obj:`list` < :obj:`int` >) shape
+        self.shape = shape
+        # : (:obj:`str`): data type
+        self.dtype = dtype
+        #: (:obj:`list` < :obj:`int` >) maximal shape
+        self.maxshape = maxshape
+
+    def __setitem__(self, key, source):
+        """ add target field to layout
+
+        :param key: slide
+        :type key: :obj:`tuple`
+        :param source: target field view
+        :type source: :class:`H5PYTargetFieldView`
+        """
+        self.add(key, source)
+
+    def add(self, key, source, sourcekey=None, shape=None):
+        """ add target field to layout
+
+        :param key: slide
+        :type key: :obj:`tuple`
+        :param source: target field view
+        :type source: :class:`H5PYTargetFieldView`
+        :param sourcekey: slide or selection
+        :type sourcekey: :obj:`tuple`
+        :param shape: target shape in the layout
+        :type shape: :obj:`tuple`
+        """
+        if shape is None:
+            shape = list(source.shape or [])
+            if hasattr(key, "__len__"):
+                size = len(key)
+                while len(shape) < size:
+                    shape.insert(0, 1)
+        lds = h5cpp.dataspace.Simple(tuple(shape))
+        selection = None
+        if key is not None and key != filewriter.FTHyperslab():
+            selection = _slice2selection(key, shape)
+            lview = h5cpp.dataspace.View(lds, selection)
+        else:
+            lview = h5cpp.dataspace.View(lds)
+        sds = h5cpp.dataspace.Simple(tuple(shape))
+        if sourcekey is not None and sourcekey != filewriter.FTHyperslab():
+            srcsel = _slice2selection(sourcekey, source.shape)
+            eview = h5cpp.dataspace.View(sds, srcsel)
+        elif selection is not None:
+            usel = unlimited_selection(selection, shape)
+            if usel is not None:
+                eview = h5cpp.dataspace.View(sds, usel)
+            else:
+                eview = h5cpp.dataspace.View(sds)
+        else:
+            eview = h5cpp.dataspace.View(sds)
+        fname = source.filename
+        path = h5cpp.Path(source.fieldpath)
+        self._h5object.add(h5cpp.property.VirtualDataMap(
+            lview, fname, path, eview))
+
+
+class H5CppTargetFieldView(filewriter.FTTargetFieldView):
+
+    """ target field for VDS """
+
+    def __init__(self, filename, fieldpath, shape, dtype=None, maxshape=None):
+        """ constructor
+
+        :param filename: file name
+        :type filename: :obj:`str`
+        :param fieldpath: nexus field path
+        :type fieldpath: :obj:`str`
+        :param shape: shape
+        :type shape: :obj:`list` < :obj:`int` >
+        :param dtype: attribute type
+        :type dtype: :obj:`str`
+        :param maxshape: shape
+        :type maxshape: :obj:`list` < :obj:`int` >
+        """
+        filewriter.FTTargetFieldView.__init__(self, None)
+        #: (:obj:`str`) directory and file name
+        self.filename = filename
+        #: (:obj:`str`) nexus field path
+        self.fieldpath = fieldpath
+        #: (:obj:`list` < :obj:`int` >) shape
+        self.shape = shape
+        # : (:obj:`str`): data type
+        self.dtype = dtype
+        #: (:obj:`list` < :obj:`int` >) maximal shape
+        self.maxshape = maxshape
+
+
 class H5CppDeflate(H5CppDataFilter):
-    pass
+
+    """ deflate filter """
 
 
 class H5CppAttributeManager(filewriter.FTAttributeManager):
@@ -1089,7 +1454,7 @@ class H5CppAttributeManager(filewriter.FTAttributeManager):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -1170,6 +1535,14 @@ class H5CppAttributeManager(filewriter.FTAttributeManager):
         return H5CppAttribute(
             self._h5object.__getitem__(name), self.parent)
 
+    def names(self):
+        """ key values
+
+        :returns: attribute names
+        :rtype: :obj:`list` <:obj:`str`>
+        """
+        return [att.name for att in self._h5object]
+
     def close(self):
         """ close attribure manager
         """
@@ -1199,7 +1572,7 @@ class H5CppAttribute(filewriter.FTAttribute):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -1419,6 +1792,8 @@ class H5CppAttribute(filewriter.FTAttribute):
         """
         if hasattr(self._h5object.dataspace, "current_dimensions"):
             return self._h5object.dataspace.current_dimensions
+        if self._h5object.dataspace.type == h5cpp.dataspace.Type.SCALAR:
+            return ()
         else:
             return (1,)
 
