@@ -88,7 +88,8 @@ class HistogramHLUTWidget(_pg.widgets.GraphicsView.GraphicsView):
         :param kargs:  HistogramHLUTItem parameter dictionary
         :type kargs: :obj:`dict` < :obj:`str`, :obj:`any`>
         """
-        background = kargs.get('background', 'default')
+        background = kargs.pop('background', 'default')
+
         _pg.widgets.GraphicsView.GraphicsView.__init__(
             self, parent, useOpenGL=False, background=background)
         #: (:class:`HistogramHLUTItem`) histogram item
@@ -289,7 +290,8 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
             self.gradient.addMenuActions()
         self.gradient.setOrientation('bottom')
         self.gradient.loadPreset('grey')
-
+        #: (:obj:`list`) buffer for removed gradients
+        self.__oldgradient = []
         #: (:class:`pyqtgraph.graphicsItems.LinearRegionItem.LinearRegionItem`)
         #:    linear region item
         self.region = _pg.graphicsItems.LinearRegionItem.LinearRegionItem(
@@ -364,7 +366,11 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
         self.gradient.removeAction.triggered.disconnect(
             self._emitRemoveGradientRequested)
         self.gradient.hide()
+        if hasattr(self.gradient, "prepareGeometryChange"):
+            self.gradient.prepareGeometryChange()
         self.layout.removeItem(self.gradient)
+        self.gradient.close()
+        self.__oldgradient.append(self.gradient)
         self.gradient = GradientEditorItemWS()
         if self.__expertmode:
             self.gradient.addMenuActions()
