@@ -109,6 +109,8 @@ class IntensityParameters(object):
         """
         #: (:obj:`bool`) do background substraction
         self.dobkgsubtraction = False
+        #: (:obj:`bool`) do brightfield substraction
+        self.dobfsubtraction = False
         #: (:obj:`bool`) calculate statistics without scaling
         self.statswoscaling = True
         #: (:obj:`str`) intensity scaling
@@ -157,11 +159,11 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         #:            axes parameters
         self.__axes = AxesParameters()
         #: (:class:`lavuelib.imageDisplayWidget.AxesParameters`)
-        #:            axes parameters
+        #:            down-sampling and range window axes parameters
         self.__wraxes = AxesParameters()
         #: (:class:`lavuelib.imageDisplayWidget.AxesParameters`)
-        #:            down-sampling and range window axes parameters
-        self.__polaraxes = AxesParameters()
+        #:            tool axes parameters
+        self.__toolaxes = AxesParameters()
 
         #: (:class:`lavuelib.imageDisplayWidget.IntensityParameters`)
         #:                  intensity parameters
@@ -341,7 +343,7 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         return "%s,%s,%s,%s" % (
             vr0[0], vr1[0], vr0[1] - vr0[0], vr1[1] - vr1[0])
 
-    def __setScale(self, position=None, scale=None, update=True, polar=False,
+    def __setScale(self, position=None, scale=None, update=True, tool=False,
                    force=False, wrenabled=None, wrupdate=True):
         """ set axes scales
 
@@ -351,8 +353,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         :type scale: [:obj:`float`, :obj:`float`]
         :param update: update scales on image
         :type update: :obj:`bool`
-        :param polar: update polar scale
-        :type polar: :obj:`bool`
+        :param tool: update tool scale
+        :type tool: :obj:`bool`
         :param force: force rescaling
         :type force: :obj:`bool`
         :param wrenabled: down-sampling rescale
@@ -364,8 +366,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
             self.__wraxes.enabled = wrenabled
         else:
             wrenabled = self.__wraxes.enabled
-        if polar:
-            axes = self.__polaraxes
+        if tool:
+            axes = self.__toolaxes
         elif self.__axes.enabled:
             axes = self.__axes
         else:
@@ -382,7 +384,7 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                 return
         axes.position = position
         axes.scale = scale
-        if wrupdate and not polar:
+        if wrupdate and not tool:
             self.__wraxes.position = position
             self.__wraxes.scale = scale
             self.__axes.position = position
@@ -413,7 +415,7 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         if self.sceneObj.rawdata is not None and update:
             self.autoRange()
 
-    def setPolarScale(self, position=None, scale=None):
+    def setToolScale(self, position=None, scale=None):
         """ set axes scales
 
         :param position: start position of axes
@@ -423,8 +425,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         :param update: update scales on image
         :type updatescale: :obj:`bool`
         """
-        self.__polaraxes.position = position
-        self.__polaraxes.scale = scale
+        self.__toolaxes.position = position
+        self.__toolaxes.scale = scale
 
     def scale(self, useraxes=True, noNone=False):
         """ provides scale and position of the axes
@@ -456,14 +458,14 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
 
         return position[0], position[1], scale[0], scale[1]
 
-    def __resetScale(self, polar=False):
+    def __resetScale(self, tool=False):
         """ reset axes scales
 
-        :param polar: update polar scale
-        :type polar: :obj:`bool`
+        :param tool: update tool scale
+        :type tool: :obj:`bool`
         """
-        if polar:
-            axes = self.__polaraxes
+        if tool:
+            axes = self.__toolaxes
         elif self.__axes.enabled:
             axes = self.__axes
         else:
@@ -573,9 +575,18 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         if ilabel is None:
             scaling = self.__intensity.scaling \
                 if not self.__intensity.statswoscaling else "linear"
-            if self.__intensity.dobkgsubtraction:
+            if self.__intensity.dobkgsubtraction and \
+               not self.__intensity.dobfsubtraction:
                 ilabel = "%s(intensity-background)" % (
                     scaling if scaling != "linear" else "")
+            elif (self.__intensity.dobkgsubtraction and
+                  self.__intensity.dobfsubtraction):
+                ilabel = "%s((intensity-DF)/(BF-DF))" % (
+                    scaling if scaling != "linear" else "")
+            elif (not self.__intensity.dobkgsubtraction and
+                  self.__intensity.dobfsubtraction):
+                ilabel = "%s(intensity/BF)" % (
+                    scaling if scaling != "linear" else "")
             else:
                 if scaling == "linear":
                     ilabel = "intensity"
@@ -598,6 +609,13 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         """
         return (self.__axes.xunits, self.__axes.yunits)
 
+    def axestext(self):
+        """ return axes text
+        :returns: x,y text
+        :rtype: (:obj:`str`, :obj:`str`)
+        """
+        return (self.__axes.xtext, self.__axes.ytext)
+
     def scaledxy(self, x, y, useraxes=True):
         """ provides scaled x,y positions
 
@@ -818,27 +836,27 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                 doreset = self.__axes.enabled
             self.__axes.enabled = parameters.scale
             # self.__wraxes.enabled = parameters.scale
-        if parameters.polarscale is not None:
-            doreset = doreset or parameters.polarscale
-            if self.__polaraxes.enabled and not parameters.polarscale:
+        if parameters.toolscale is not None:
+            doreset = doreset or parameters.toolscale
+            if self.__toolaxes.enabled and not parameters.toolscale:
                 doreset = True
                 rescale = True
-            self.__polaraxes.enabled = parameters.polarscale
+            self.__toolaxes.enabled = parameters.toolscale
 
         for ext in self.__extensions.values():
             ext.show(parameters)
 
         if doreset:
-            self.__resetScale(polar=parameters.polarscale)
+            self.__resetScale(tool=parameters.toolscale)
         if self.__wraxes.enabled:
             self.__setScale(
                 self.__wraxes.position, self.__wraxes.scale, force=rescale)
         if parameters.scale is True or rescale:
             self.__setScale(
                 self.__axes.position, self.__axes.scale, force=rescale)
-        if parameters.polarscale is True:
+        if parameters.toolscale is True:
             self.__setScale(
-                self.__polaraxes.position, self.__polaraxes.scale, polar=True)
+                self.__toolaxes.position, self.__toolaxes.scale, tool=True)
 
     @QtCore.pyqtSlot(bool)
     def emitAspectLockedToggled(self, status):
@@ -897,6 +915,52 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
             return True
         return False
 
+    def updateTicks(self, record):
+        """ update Ticks values
+
+        :param record: dict record with the tick parameters:
+                       "position" : [x, y]
+                       "scale" : [sx, sy]
+                       "xtext" : xlabel
+                       "ytext" : ylabel
+                       "xunits" : xunits
+                       "yunits" : yunits
+        :type record: :obj:`dict`<:obj:`str`, `any`>
+        """
+        update = False
+        if "xtext" in record.keys():
+            self.__axes.xtext = str(record["xtext"] or "")
+            update = True
+        if "ytext" in record.keys():
+            self.__axes.ytext = str(record["ytext"] or "")
+            update = True
+        if "xunits" in record.keys():
+            self.__axes.xunits = str(record["xunits"] or "")
+            update = True
+        if "yunits" in record.keys():
+            self.__axes.yunits = str(record["yunits"] or "")
+            update = True
+
+        if "position" in record.keys() or "scale" in record.keys():
+            update = True
+            if "scale" not in record.keys():
+                scale = self.__axes.scale
+            else:
+                scale = record["scale"]
+
+            if "position" not in record.keys():
+                position = self.__axes.position
+            else:
+                position = record["position"]
+            try:
+                self.__setScale(position, scale, wrupdate=False)
+            except Exception as e:
+                logger.warning(str(e))
+                # print(str(e))
+
+        if update:
+            self.updateImage(self.__data, self.sceneObj.rawdata)
+
     def rawData(self):
         """ provides the raw data
 
@@ -983,6 +1047,14 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         """
         self.__intensity.dobkgsubtraction = state
 
+    def setDoBFSubtraction(self, state):
+        """ sets do brightfield subtraction flag
+
+        :param status: do brightfield subtraction flag
+        :type status: :obj:`bool`
+        """
+        self.__intensity.dobfsubtraction = state
+
     def image(self):
         """ provides imageItem object
 
@@ -1063,15 +1135,19 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         elif self.__wraxes.enabled is True:
             self.__setScale(self.__wraxes.position,
                             self.__wraxes.scale)
-        if self.__polaraxes.enabled is True:
+        if self.__toolaxes.enabled is True:
             self.__setScale(
-                self.__polaraxes.position, self.__polaraxes.scale, polar=True)
+                self.__toolaxes.position, self.__toolaxes.scale, tool=True)
 
     def autoRange(self):
         """ sets auto range
         """
-        self.__viewbox.autoRange()
-        self.__viewbox.enableAutoRange('xy', True)
+        try:
+            self.__viewbox.autoRange()
+            self.__viewbox.enableAutoRange('xy', True)
+        except Exception as e:
+            logger.warning(str(e))
+            # print(str(e))
 
     def rangeWindowEnabled(self):
         """ provide info if range window enabled
