@@ -56,11 +56,6 @@ except ImportError:
 
 #: (:obj:`dict` <:obj:`str`, :obj:`module`> ) nexus writer modules
 WRITERS = {}
-try:
-    from . import pniwriter
-    WRITERS["pni"] = pniwriter
-except Exception:
-    pass
 try:
     from . import h5pywriter
     WRITERS["h5py"] = h5pywriter
@@ -86,7 +81,7 @@ class NexusFieldHandler(object):
 
         :param fname: file name
         :type fname: :obj:`str`
-        :param writer: h5 writer module: "pni" or "h5py"
+        :param writer: h5 writer module: "h5cpp" or "h5py"
         :type writer: :obj:`str`
         """
         #: (:obj:`any`) module image object
@@ -100,29 +95,29 @@ class NexusFieldHandler(object):
         self.__fields = {}
         # (:class:`lavuelib.filewriter.root`) nexus file root
         self.__root = None
+        # (:class:`lavuelib.filewriter.FTFile') nexus file object
+        self.__fl = None
         #: (:obj:`str`) json dictionary with metadata or empty string
         self.__metadata = ""
 
         if not writer:
             if "h5cpp" in WRITERS.keys():
                 writer = "h5cpp"
-            elif "h5py" in WRITERS.keys():
-                writer = "h5py"
             else:
-                writer = "pni"
+                writer = "h5py"
         if writer not in WRITERS.keys():
             raise Exception("Writer '%s' cannot be opened" % writer)
         wrmodule = WRITERS[writer.lower()]
         if fname:
             try:
-                fl = filewriter.open_file(
+                self.__fl = filewriter.open_file(
                     fname, writer=wrmodule, readonly=True,
                     libver='latest',
                     swmr=(True if writer in ["h5py", "h5cpp"] else False)
                 )
             except Exception:
                 try:
-                    fl = filewriter.open_file(
+                    self.__fl = filewriter.open_file(
                         fname, writer=wrmodule, readonly=True)
                 except Exception:
                     raise Exception("File '%s' cannot be opened \n" % (fname))
@@ -130,7 +125,52 @@ class NexusFieldHandler(object):
                 #     raise Exception("File '%s' cannot be opened %s\n"
                 #                % (fname, str(e)))
 
-            self.__root = fl.root()
+            self.__root = self.__fl.root()
+
+    def frombuffer(self, membuffer, fname=None, writer=None):
+        """ constructor
+
+        :param membuffer: memory buffer
+        :type membuffer: :obj:`bytes` or :obj:`io.BytesIO`
+        :param fname: file name
+        :type fname: :obj:`str`
+        :param writer: h5 writer module: "h5py" or "h5py"
+        :type writer: :obj:`str`
+        """
+        if fname is not None:
+            self.__fname = fname
+
+        if not writer:
+            if "h5cpp" in WRITERS.keys() and \
+               WRITERS["h5cpp"].is_image_file_supported():
+                writer = "h5cpp"
+            elif "h5py" in WRITERS.keys() and \
+                 WRITERS["h5py"].is_image_file_supported():
+                writer = "h5py"
+            elif "h5cpp" in WRITERS.keys():
+                writer = "h5cpp"
+            else:
+                writer = "h5py"
+        if writer not in WRITERS.keys():
+            raise Exception("Writer '%s' cannot be opened" % writer)
+        wrmodule = WRITERS[writer.lower()]
+        try:
+            self.__fl = filewriter.load_file(
+                membuffer,
+                fname=self.__fname, writer=wrmodule, readonly=True,
+                libver='latest',
+                swmr=(True if writer in ["h5py", "h5cpp"] else False)
+            )
+        except Exception:
+            try:
+                self.__fl = filewriter.load_file(
+                    membuffer,
+                    fname, writer=wrmodule, readonly=True)
+            except Exception:
+                raise Exception(
+                    "File '%s' cannot be loaded \n" % (self.__fname))
+
+        self.__root = self.__fl.root()
 
     def findImageFields(self):
         """ provides a dictionary with of all image fields
@@ -160,11 +200,8 @@ class NexusFieldHandler(object):
         """adds the node into the description list
 
         :param node: nexus node
-        :type node: :class:`pni.io.nx.h5.nxfield` or \
-                    :class:`pni.io.nx.h5.nxgroup` or \
-                    :class:`pni.io.nx.h5.nxlink` or \
-                    :class:`pni.io.nx.h5.nxattribute` or \
-                    :class:`pni.io.nx.h5.nxroot`
+        :type node: :class:`lavuelib.filewriter.FTField` or \
+                    :class:`lavuelib.filewriter.FTGroup`
         :param path: path of the link target or `None`
         :type path: :obj:`str`
         """
@@ -193,11 +230,8 @@ class NexusFieldHandler(object):
         """parses the node and add it into the description list
 
         :param node: nexus node
-        :type node: :class:`pni.io.nx.h5.nxfield` or \
-                    :class:`pni.io.nx.h5.nxgroup` or \
-                    :class:`pni.io.nx.h5.nxlink` or \
-                    :class:`pni.io.nx.h5.nxattribute` or \
-                    :class:`pni.io.nx.h5.nxroot`
+        :type node: :class:`lavuelib.filewriter.FTField` or \
+                    :class:`lavuelib.filewriter.FTGroup` or
         :param path: path of the link target or `None`
         :type path: :obj:`str`
         """
@@ -217,16 +251,12 @@ class NexusFieldHandler(object):
             finally:
                 pass
 
-    def getNode(self, field):
+    def getNode(self, field=None):
         """ get node
         :param field: field path
         :type field: :obj:`str`
         :returns: nexus field node
-        :rtype: :class:`pni.io.nx.h5.nxfield` or \
-                :class:`pni.io.nx.h5.nxgroup` or \
-                :class:`pni.io.nx.h5.nxlink` or \
-                :class:`pni.io.nx.h5.nxattribute` or \
-                :class:`pni.io.nx.h5.nxroot`
+        :rtype: :class:`lavuelib.filewriter.FTField`
         """
         node = None
         if field is not None:
@@ -235,18 +265,16 @@ class NexusFieldHandler(object):
             for name in sfield:
                 if name:
                     node = node.open(name)
+        else:
+            node = self.__fl.default_field()
         return node
 
     @classmethod
-    def getLastFrame(cls, node, growing=0, refresh=True):
+    def getFrameCount(cls, node, growing=0, refresh=True):
         """ provides the last frame number
 
         :param node: nexus field node
-        :type node: :class:`pni.io.nx.h5.nxfield` or \
-                    :class:`pni.io.nx.h5.nxgroup` or \
-                    :class:`pni.io.nx.h5.nxlink` or \
-                    :class:`pni.io.nx.h5.nxattribute` or \
-                    :class:`pni.io.nx.h5.nxroot`
+        :type node: :class:`lavuelib.filewriter.FTField`
         :param growing: growing dimension
         :type growing: :obj:`int`
         :param refresh: refresh image node
@@ -276,7 +304,6 @@ class NexusFieldHandler(object):
                   'x_pixel_size', 'y_pixel_size',
                   'beam_center_x', 'beam_center_y']
         names = pgroup.names()
-
         for nm in mnames:
             if nm in names and nm not in metadata.keys():
                 fld = pgroup.open(nm)
@@ -284,22 +311,35 @@ class NexusFieldHandler(object):
                 if isinstance(value, np.ndarray):
                     continue
                 try:
-                    units = cls.extract(fld.attributes["units"].read())
-                    if isinstance(units, np.ndarray):
-                        continue
-                    if units:
-                        metadata[nm] = (value, units)
+                    atts = [at.name for at in fld.attributes]
+                    if "units" in atts:
+                        units = cls.extract(fld.attributes["units"].read())
+                        if isinstance(units, np.ndarray):
+                            continue
+                        if units:
+                            metadata[nm] = (value, units)
+                        else:
+                            metadata[nm] = value
                     else:
                         metadata[nm] = value
-                except Exception:
+                except Exception as e:
+                    # print(str(e))
+                    logger.warning(str(e))
                     metadata[nm] = value
         lfld = pgroup.open_link(node.name)
-        file_path, lpath = str(lfld.target_path).rsplit(":/", 1)
+        slpath = str(lfld.target_path).rsplit(":/", 1)
+        if len(slpath) > 1:
+            file_path = slpath[0]
+            lpath = slpath[1]
+        else:
+            file_path = None
+            lpath = slpath[0]
         opath = "/".join([gr.split(":")[0]
                           for gr in node.path.split("/") if gr != '.'])
         lpath = lpath.replace("//", "/")
         lfile = lfld.getfilename(lfld)
-        if maxrec and lfile == file_path and str(lpath) != str(opath):
+        if lfile is not None and \
+           maxrec and lfile == file_path and str(lpath) != str(opath):
             try:
                 root = None
                 par = pgroup
@@ -335,11 +375,7 @@ class NexusFieldHandler(object):
         """parses the field and add it into the description list
 
         :param node: nexus field node
-        :type node: :class:`pni.io.nx.h5.nxfield` or \
-                    :class:`pni.io.nx.h5.nxgroup` or \
-                    :class:`pni.io.nx.h5.nxlink` or \
-                    :class:`pni.io.nx.h5.nxattribute` or \
-                    :class:`pni.io.nx.h5.nxroot`
+        :type node: :class:`lavuelib.filewriter.FTField`
         :param frame: frame to take, the last one is -1
         :type frame: :obj:`int`
         :param growing: growing dimension
@@ -354,6 +390,8 @@ class NexusFieldHandler(object):
         if node:
             shape = node.shape
         if shape:
+            if frame is None:
+                return node.read()
             if len(shape) == 2:
                 return node[...]
             elif len(shape) == 3:
@@ -451,16 +489,18 @@ class CBFLoader(object):
     """ CBF loader """
 
     @classmethod
-    def metadata(cls, flbuffer):
+    def metadata(cls, flbuffer, premeta=None):
         """ extract header_contents from CBF file image data
 
         :param flbuffer: numpy array with CBF file image data
         :type flbuffer: :class:`numpy.ndarray` or :obj:`str`
+        :param premeta: metadata to be merged
+        :type premeta: :obj:`dict`  <:obj:`str`, :obj:`any`>
         :returns: metadata dictionary
         :rtype: :class:`numpy.ndarray`
         """
-        headerstart = '_array_data.header_convention'
-        headerend = '_array_data.data'
+        headerstart = b'_array_data.header_convention'
+        headerend = b'_array_data.data'
         mdata = {}
         try:
             if hasattr(flbuffer, "tostring"):
@@ -551,9 +591,11 @@ class CBFLoader(object):
                 # print(str(e))
                 logger.warning(str(e))
         if mdata:
+            if premeta:
+                mdata.update(premeta)
             return json.dumps(mdata)
         else:
-            return ""
+            return json.dumps(premeta) if premeta else ""
 
     @classmethod
     def load(cls, flbuffer):
