@@ -43,6 +43,7 @@ from . import imageSource as isr
 from . import toolWidget
 from . import memoExportDialog
 from . import sardanaUtils
+from .sardanaUtils import debugmethod
 
 # _VMAJOR, _VMINOR, _VPATCH = _pg.__version__.split(".") \
 #     if _pg.__version__ else ("0", "9", "0")
@@ -103,6 +104,10 @@ class ImageWidget(QtGui.QWidget):
     clearBottomPlotClicked = QtCore.pyqtSignal()
     #: (:class:`pyqtgraph.QtCore.pyqtSignal`) scales changed signal
     scalesChanged = QtCore.pyqtSignal()
+    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) colors changed signal
+    colorsChanged = QtCore.pyqtSignal(str)
+    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) tool configuration changed signal
+    toolConfigurationChanged = QtCore.pyqtSignal()
 
     def __init__(self, parent=None, tooltypes=None, settings=None,
                  rgbtooltypes=None):
@@ -137,6 +142,9 @@ class ImageWidget(QtGui.QWidget):
         #:           tool names
         self.__rgbtoolwidgets = {}
 
+        #: (:class:`pyqtgraph.QtCore.QMutex`) mutex lock for CB
+        self.__mutex = QtCore.QMutex()
+
         #: (:class:`lavuelib.settings.Settings`) settings
         self.__settings = settings
         #: (:class:`lavuelib.controllerClient.ControllerClient`)
@@ -159,6 +167,16 @@ class ImageWidget(QtGui.QWidget):
         self.__data = None
         #: (:class:`numpy.ndarray`) raw data to cut plots
         self.__rawdata = None
+        #: (:obj:`bool`) apply mask
+        self.__applymask = False
+        #: (:class:`numpy.ndarray`) mask image indices
+        self.__maskindices = None
+        #: (:class:`numpy.ndarray`) mask image value indices
+        self.__maskvalueindices = None
+        #: (:obj:`float`) file name
+        self.__maskvalue = None
+        #: (:obj:`str`) image name
+        self.__imagename = None
 
         #: ( ( :obj:`bool`, :obj:`bool`,:obj:`bool`) )
         #        selected (transpose, leftright-flip, updown-flip )
@@ -182,11 +200,14 @@ class ImageWidget(QtGui.QWidget):
                 displayExtensions.MeshExtension,
                 displayExtensions.MaximaExtension,
                 displayExtensions.VHBoundsExtension,
+                displayExtensions.RegionsExtension,
             ]
         )
 
         #: (:class:`pyqtgraph.PlotWidget`) bottom 1D plot widget
         self.__bottomplot = memoExportDialog.MemoPlotWidget(self)
+        self.__bottomplot.addLegend()
+        self.__bottomplot.plotItem.legend.hide()
 
         #: (:class:`pyqtgraph.PlotWidget`) right 1D plot widget
         self.__rightplot = memoExportDialog.MemoPlotWidget(self)
@@ -280,6 +301,21 @@ class ImageWidget(QtGui.QWidget):
         if self.__tangoclient:
             self.__tangoclient.writeAttribute(name, value)
 
+    def writeDetectorAttributes(self):
+        """ write detector settings from ai object
+        """
+        self.writeAttribute("BeamCenterX",
+                            float(self.__settings.centerx))
+        self.writeAttribute("BeamCenterY",
+                            float(self.__settings.centery))
+        self.writeAttribute("Energy", float(self.__settings.energy))
+        self.writeAttribute("DetectorDistance",
+                            float(self.__settings.detdistance))
+        self.writeAttribute("PixelSizeX",
+                            float(self.__settings.pixelsizex))
+        self.writeAttribute("PixelSizeY",
+                            float(self.__settings.pixelsizey))
+
     @QtCore.pyqtSlot()
     def writeDetectorROIsAttribute(self):
         """ writes DetectorROIsattribute value of device
@@ -417,7 +453,7 @@ class ImageWidget(QtGui.QWidget):
         self.__ui.upperPlotSplitter.moveSplitter(pos, index)
         self.__connectsplitters()
 
-    def onedbottomplot(self, clear=False):
+    def onedbottomplot(self, clear=False, name=None):
         """ creates 1d bottom plot
 
         :param clear: clear flag
@@ -425,7 +461,24 @@ class ImageWidget(QtGui.QWidget):
         :returns: 1d bottom plot
         :rtype: :class:`pyqtgraph.PlotDataItem`
         """
-        return self.__bottomplot.plot(clear=clear)
+        return self.__bottomplot.plot(clear=clear, name=name)
+
+    def onedshowlegend(self, show=True):
+        """ shows/hides 1d bottom plot legend
+
+        :param status: show flag
+        :type status: :obj:`bool`
+        :returns: 1d bottom plot
+        :rtype: :class:`pyqtgraph.PlotDataItem`
+        """
+        if show:
+            self.__bottomplot.plotItem.legend.show()
+        else:
+            legend = self.__bottomplot.plotItem.legend
+            its = [it[1].text for it in legend.items]
+            for it in its:
+                legend.removeItem(it)
+            legend.hide()
 
     def bottomplotShowMenu(self, freeze=False, clear=False):
         """ shows freeze or/and clean action in the menu
@@ -437,6 +490,19 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__bottomplot.showMenu(freeze, clear)
 
+    def bottomplotStretch(self, stretch=1):
+        """ stretches the bottom plot
+
+        :param stretch: stretch factor
+        :type stretch: :obj:`int`
+        """
+        self.__ui.plotSplitter.setStretchFactor(1, stretch)
+        if stretch >= 1000:
+            self.__ui.plotSplitter.setStretchFactor(0, 0)
+            self.__ui.plotSplitter.setSizes([0, 50])
+        else:
+            self.__ui.plotSplitter.setStretchFactor(0, 1)
+
     def onedrightplot(self, clear=False):
         """ creates 1d right plot
 
@@ -511,6 +577,7 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__settings
 
+    @debugmethod
     def __connecttool(self):
         """ connect current tool widget
         """
@@ -523,7 +590,8 @@ class ImageWidget(QtGui.QWidget):
                 signal.connect(slot)
             self.__currenttool.activate()
 
-    def __disconnecttool(self):
+    @debugmethod
+    def disconnecttool(self):
         """ disconnect current tool widget
         """
         if self.__currenttool:
@@ -537,7 +605,7 @@ class ImageWidget(QtGui.QWidget):
                 except Exception as e:
                     # print(str(e))
                     logger.warning(str(e))
-                self.__currenttool.deactivate()
+            self.__currenttool.deactivate()
 
     def updateMetaData(self, axisscales=None, axislabels=None,
                        rescale=False):
@@ -571,6 +639,21 @@ class ImageWidget(QtGui.QWidget):
         self.roiCoordsChanged.emit()
         self.roiNumberChanged.emit(rid)
 
+    @QtCore.pyqtSlot(int)
+    def updateRegions(self, points=None, rid=None):
+        """ update Ranges
+
+        :param points: roi coordinates
+        :type points: :obj:`list` <  :obj:`list` <  :obj:`list`
+                  < (:obj:`float`, :obj:`float`) > > >
+        :param rid: region id
+        :type rid: :obj:`int`
+        """
+        self.__displaywidget.extension('regions').updateRegions(points, rid)
+        # self.applyTipsChanged.emit(rid)
+        # self.roiCoordsChanged.emit()
+        # self.roiNumberChanged.emit(rid)
+
     @QtCore.pyqtSlot(int)
     def updateCuts(self, cid, coords=None):
         """ update Cuts
@@ -593,56 +676,59 @@ class ImageWidget(QtGui.QWidget):
         """
         return str(self.__ui.toolComboBox.currentText())
 
-    @QtCore.pyqtSlot(int)
+    @debugmethod
     @QtCore.pyqtSlot()
     def showCurrentTool(self):
         """ shows the current tool
         """
-        text = self.__ui.toolComboBox.currentText()
-        stwg = None
-        self.__ui.rgbtoolComboBox.hide()
-        self.__ui.toolComboBox.show()
-        for nm, twg in self.__rgbtoolwidgets.items():
-            twg.hide()
-        for nm, twg in self.__toolwidgets.items():
-            if text == nm:
-                stwg = twg
-            else:
+        with QtCore.QMutexLocker(self.__mutex):
+            text = self.__ui.toolComboBox.currentText()
+            stwg = None
+            self.__ui.rgbtoolComboBox.hide()
+            self.__ui.toolComboBox.show()
+            for nm, twg in self.__rgbtoolwidgets.items():
                 twg.hide()
-        self.__disconnecttool()
-        self.__currenttool = stwg
-        if stwg is not None:
-            stwg.show()
-            self.updateinfowidgets(stwg.parameters)
+            for nm, twg in self.__toolwidgets.items():
+                if text == nm:
+                    stwg = twg
+                else:
+                    twg.hide()
+            self.disconnecttool()
+            self.__currenttool = stwg
+            if stwg is not None:
+                stwg.show()
+                self.updateinfowidgets(stwg.parameters)
 
-        self.__connecttool()
-        self.currentToolChanged.emit(text)
+            self.__connecttool()
+            self.currentToolChanged.emit(text)
 
-    @QtCore.pyqtSlot(int)
+    @debugmethod
     @QtCore.pyqtSlot()
     def showCurrentRGBTool(self):
         """ shows the current tool
         """
-        text = self.__ui.rgbtoolComboBox.currentText()
-        stwg = None
-        self.__ui.toolComboBox.hide()
-        self.__ui.rgbtoolComboBox.show()
-        for nm, twg in self.__toolwidgets.items():
-            twg.hide()
-        for nm, twg in self.__rgbtoolwidgets.items():
-            if text == nm:
-                stwg = twg
-            else:
+        with QtCore.QMutexLocker(self.__mutex):
+            text = self.__ui.rgbtoolComboBox.currentText()
+            stwg = None
+            self.__ui.toolComboBox.hide()
+            self.__ui.rgbtoolComboBox.show()
+            for nm, twg in self.__toolwidgets.items():
                 twg.hide()
-        self.__disconnecttool()
-        self.__currenttool = stwg
-        if stwg is not None:
-            stwg.show()
-            self.updateinfowidgets(stwg.parameters)
+            for nm, twg in self.__rgbtoolwidgets.items():
+                if text == nm:
+                    stwg = twg
+                else:
+                    twg.hide()
+            self.disconnecttool()
+            self.__currenttool = stwg
+            if stwg is not None:
+                stwg.show()
+                self.updateinfowidgets(stwg.parameters)
 
-        self.__connecttool()
-        self.currentToolChanged.emit(text)
+            self.__connecttool()
+            self.currentToolChanged.emit(text)
 
+    # @debugmethod
     def showTool(self, text):
         """ shows the current tool
         """
@@ -652,7 +738,7 @@ class ImageWidget(QtGui.QWidget):
                 stwg = twg
             else:
                 twg.hide()
-        self.__disconnecttool()
+        self.disconnecttool()
         self.__currenttool = stwg
         if stwg is not None:
             stwg.show()
@@ -780,13 +866,15 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.transformations()
 
-    def plot(self, array, rawarray=None):
+    def plot(self, array, rawarray=None, imagename=None):
         """ plots the image
 
         :param array: 2d image array
         :type array: :class:`numpy.ndarray`
         :param rawarray: 2d raw image array
         :type rawarray: :class:`numpy.ndarray`
+        :param imagename: image name
+        :type imagename: :obj:`str`
         """
         if array is None:
             return
@@ -795,14 +883,27 @@ class ImageWidget(QtGui.QWidget):
         barrays = None
         self.__data = array
         self.__rawdata = rawarray
+        self.__imagename = imagename
         if self.__currenttool:
             barrays = self.__currenttool.beforeplot(array, rawarray)
         self.__displaywidget.updateImage(
-            barrays[0] if barrays is not None else array,
-            barrays[1] if barrays is not None else rawarray)
+            barrays[0] if barrays is not None else self.__data,
+            barrays[1] if barrays is not None else self.__rawdata)
         if self.__currenttool:
             self.__currenttool.afterplot()
 
+    def updateImage(self, array=None, rawarray=None):
+        """ update the image
+
+        :param array: 2d image array
+        :type array: :class:`numpy.ndarray`
+        :param rawarray: 2d raw image array
+        :type rawarray: :class:`numpy.ndarray`
+        """
+        self.__displaywidget.updateImage(
+            array if array is not None else self.__data,
+            rawarray if rawarray is not None else self.__rawdata)
+
     @QtCore.pyqtSlot(int)
     def setAutoLevels(self, autolevels):
         """ sets auto levels
@@ -928,6 +1029,22 @@ class ImageWidget(QtGui.QWidget):
         :rtype: :obj:`bool`
         """
         self.__displaywidget.setTicks()
+        self.emitTCC()
+
+    def updateTicks(self, record):
+        """ update Ticks values
+
+        :param record: dict record with the tick parameters:
+                       "position" : [x, y]
+                       "scale" : [sx, sy]
+                       "xtext" : xlabel
+                       "ytext" : ylabel
+                       "xunits" : xunits
+                       "yunits" : yunits
+        :type record: :obj:`dict`<:obj:`str`, `any`>
+        """
+        self.__displaywidget.updateTicks(record)
+        self.emitTCC()
 
     def image(self):
         """ provides imageItem object
@@ -937,6 +1054,13 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.image()
 
+    # @debugmethod
+    @QtCore.pyqtSlot()
+    def emitTCC(self):
+        """emits toolConfigurationChanged
+        """
+        self.toolConfigurationChanged.emit()
+
     @QtCore.pyqtSlot()
     def emitCutCoordsChanged(self):
         """emits cutCoordsChanged
@@ -1021,15 +1145,16 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.setStatsWOScaling(status)
 
-    def setROIsColors(self, colors):
-        """ sets statistics without scaling flag
+    def setColors(self, colors):
+        """ sets item colors
 
         :param colors: json list of roi colors
         :type colors: :obj:`str`
-        :returns: change status
-        :rtype: :obj:`bool`
         """
-        return self.__displaywidget.extension('rois').setROIsColors(colors)
+        for name in self.__displaywidget.extensions():
+            if hasattr(self.__displaywidget.extension(name), "setColors"):
+                self.__displaywidget.extension(name).setColors(colors)
+        self.colorsChanged.emit(colors)
 
     def setScalingType(self, scalingtype):
         """ sets intensity scaling types
@@ -1047,6 +1172,14 @@ class ImageWidget(QtGui.QWidget):
         """
         self.__displaywidget.setDoBkgSubtraction(state)
 
+    def setDoBFSubtraction(self, state):
+        """ sets do brightfield subtraction flag
+
+        :param status: do brightfield subtraction flag
+        :type status: :obj:`bool`
+        """
+        self.__displaywidget.setDoBFSubtraction(state)
+
     def setSardanaUtils(self, sardana):
         """ sets sardana utils
 
@@ -1068,7 +1201,7 @@ class ImageWidget(QtGui.QWidget):
         :rtype: :obj:`bool`
         """
         dp = None
-        if isr.PYTANGO:
+        if isr.TANGO:
             if not self.__settings.doorname:
                 self.__settings.doorname = self.__sardana.getDeviceName("Door")
             try:
@@ -1091,7 +1224,7 @@ class ImageWidget(QtGui.QWidget):
         :rtype: :obj:`list` <:obj:`str`>
         """
         elements = None
-        if isr.PYTANGO and self.__sardana:
+        if isr.TANGO and self.__sardana:
             if not self.__settings.doorname:
                 self.__settings.doorname = self.__sardana.getDeviceName("Door")
             try:
@@ -1111,7 +1244,7 @@ class ImageWidget(QtGui.QWidget):
         :return: macro runned
         :rtype: :obj:`bool`
         """
-        if isr.PYTANGO:
+        if isr.TANGO:
             if not self.__settings.doorname:
                 self.__settings.doorname = self.__sardana.getDeviceName("Door")
             try:
@@ -1139,7 +1272,7 @@ class ImageWidget(QtGui.QWidget):
     def showDoorError(self):
         """ show door error
         """
-        if isr.PYTANGO:
+        if isr.TANGO:
             if not self.__settings.doorname:
                 self.__settings.doorname = self.__sardana.getDeviceName("Door")
             try:
@@ -1169,7 +1302,7 @@ class ImageWidget(QtGui.QWidget):
         :param roispin: the current number of rois
         :type roispin: :obj:`int`
         """
-        if isr.PYTANGO:
+        if isr.TANGO:
             if self.__settings.sardana:
                 if not self.__settings.doorname:
                     self.__settings.doorname = self.__sardana.getDeviceName(
@@ -1365,7 +1498,7 @@ class ImageWidget(QtGui.QWidget):
         :param rlabel: rois aliases separated by space
         :type rlabel: :obj:`str`
         """
-        if isr.PYTANGO:
+        if isr.TANGO:
             if not self.__settings.doorname:
                 self.__settings.doorname = self.__sardana.getDeviceName("Door")
             try:
@@ -1495,6 +1628,14 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.axesunits()
 
+    def axestext(self):
+        """ return axes text
+
+        :returns: x,y text
+        :rtype: (:obj:`str`, :obj:`str`)
+        """
+        return self.__displaywidget.axestext()
+
     def roiCoords(self):
         """ provides rois coordinates
 
@@ -1615,15 +1756,18 @@ class ImageWidget(QtGui.QWidget):
         self.__displaywidget.extension('center').updateCenter(xdata, ydata)
 
     @QtCore.pyqtSlot(float, float)
-    def updatePositionMark(self, xdata, ydata):
+    def updatePositionMark(self, xdata, ydata, scaled=False):
         """ updates the position mark
 
         :param xdata: x pixel position
         :type xdata: :obj:`float`
         :param ydata: y-pixel position
         :type ydata: :obj:`float`
+        :param scaled: scaled flag
+        :type scaled: :obj:`bool`
         """
-        self.__displaywidget.extension('mark').updatePositionMark(xdata, ydata)
+        self.__displaywidget.extension('mark').updatePositionMark(
+            xdata, ydata, scaled)
 
     def setDoubleClickLock(self, status=True):
         """ sets double click lock
@@ -1633,6 +1777,7 @@ class ImageWidget(QtGui.QWidget):
         """
         self.__displaywidget.setDoubleClickLock(status)
 
+    @debugmethod
     def setTool(self, tool):
         """ sets tool from string
 
@@ -1648,6 +1793,28 @@ class ImageWidget(QtGui.QWidget):
             self.__ui.rgbtoolComboBox.setCurrentIndex(index)
             self.showCurrentRGBTool()
 
+    @debugmethod
+    def setToolConfiguration(self, config):
+        """ sets tool configuration from JSON dictionary
+
+        :param config: JSON dictionary with tool configuration
+        :type config: :obj:`str`
+        """
+        if self.__currenttool is not None:
+            try:
+                self.__currenttool.configure(config)
+            except Exception as e:
+                logger.warning(str(e))
+
+    def toolConfiguration(self):
+        """ provides tool configuration
+
+        :returns: JSON dictionary with tool configuration
+        :rtype: :obj:`str`
+        """
+        if self.__currenttool is not None:
+            return self.__currenttool.configuration()
+
     def tool(self):
         """ provices tool from string
 
@@ -1668,6 +1835,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.energy != energy:
             self.__settings.energy = energy
+            self.__settings.updateAISettings()
             self.mouseImagePositionChanged.emit()
             self.geometryChanged.emit()
 
@@ -1680,6 +1848,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.detdistance != distance:
             self.__settings.detdistance = distance
+            self.__settings.updateAISettings()
             self.mouseImagePositionChanged.emit()
             self.geometryChanged.emit()
 
@@ -1692,6 +1861,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.centerx != x:
             self.__settings.centerx = x
+            self.__settings.updateAISettings()
             self.updateCenter(
                 self.__settings.centerx, self.__settings.centery)
             self.mouseImagePositionChanged.emit()
@@ -1706,6 +1876,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.centery != y:
             self.__settings.centery = y
+            self.__settings.updateAISettings()
             self.updateCenter(
                 self.__settings.centerx, self.__settings.centery)
             self.mouseImagePositionChanged.emit()
@@ -1720,6 +1891,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.pixelsizex != x:
             self.__settings.pixelsizex = x
+            self.__settings.updateAISettings()
             self.mouseImagePositionChanged.emit()
             self.geometryChanged.emit()
 
@@ -1732,6 +1904,7 @@ class ImageWidget(QtGui.QWidget):
         """
         if self.__settings.pixelsizey != y:
             self.__settings.pixelsizey = y
+            self.__settings.updateAISettings()
             self.mouseImagePositionChanged.emit()
             self.geometryChanged.emit()
 
@@ -1783,7 +1956,7 @@ class ImageWidget(QtGui.QWidget):
         if oldcoords != coords:
             self.updateROIs(len(coords), coords)
 
-    def setPolarScale(self, position=None, scale=None):
+    def setToolScale(self, position=None, scale=None):
         """ get axes parameters
 
         :param position: start position of axes
@@ -1791,7 +1964,7 @@ class ImageWidget(QtGui.QWidget):
         :param scale: scale axes
         :type scale: [:obj:`float`, :obj:`float`]
         """
-        return self.__displaywidget.setPolarScale(position, scale)
+        return self.__displaywidget.setToolScale(position, scale)
 
     def setViewRange(self, rangelist):
         """ set view range values
@@ -1841,6 +2014,70 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.rgb()
 
+    def applyMask(self):
+        """ provides apply mask flag
+
+        :returns: True for apply mask
+        :rtype: :obj:`bool`
+        """
+        return self.__applymask
+
+    def setApplyMask(self, applymask=True):
+        """ sets apply mask flag
+
+        :params applymask: True for apply mask
+        :type applymask: :obj:`bool`
+        """
+        self.__applymask = applymask
+
+    def maskValue(self):
+        """ provides high mask value
+
+        :returns: high mask value
+        :rtype: :obj:`float`
+        """
+        return self.__maskvalue
+
+    def setMaskValue(self, maskvalue):
+        """ sets high mask value
+
+        :params applymask: high mask value
+        :type applymask: :obj:`float`
+        """
+        self.__maskvalue = maskvalue
+
+    def maskIndices(self):
+        """ provides mask image indices
+
+        :returns: mask image indices
+        :rtype: :class:`numpy.ndarray`
+        """
+        return self.__maskindices
+
+    def setMaskIndices(self, maskindices):
+        """ sets mask image indices
+
+        :params maskindices: mask image indices
+        :type maskindices: :class:`numpy.ndarray`
+        """
+        self.__maskindices = maskindices
+
+    def maskValueIndices(self):
+        """ provides mask image value indices
+
+        :returns: mask image indices
+        :rtype: :class:`numpy.ndarray`
+        """
+        return self.__maskValueIndices
+
+    def setMaskValueIndices(self, maskindices):
+        """ sets mask image indices
+
+        :params maskindices: mask image  value indices
+        :type maskindices: :class:`numpy.ndarray`
+        """
+        self.__maskValueIndices = maskindices
+
     def rangeWindowEnabled(self):
         """ provide info if range window enabled
 
@@ -1849,6 +2086,14 @@ class ImageWidget(QtGui.QWidget):
         """
         return self.__displaywidget.rangeWindowEnabled()
 
+    def imageName(self):
+        """ provide the current image name
+
+        :returns: image name
+        :rtype: :obj:`str`
+        """
+        return self.__imagename
+
     def rangeWindowScale(self):
         """ provide info range window scale
 
