@@ -35,6 +35,14 @@ from pyqtgraph import QtCore
 if sys.version_info > (3,):
     unicode = str
 
+try:
+    __import__("pyFAI")
+    #: (:obj:`bool`) pyFAI imported
+    PYFAI = True
+except ImportError:
+    #: (:obj:`bool`) pyFAI imported
+    PYFAI = False
+
 
 class Settings(object):
 
@@ -57,6 +65,8 @@ class Settings(object):
         self.showaddhisto = False
         #: (:obj:`bool`) show mask widget
         self.showmask = False
+        #: (:obj:`bool`) map image sources to color channels
+        self.imagechannels = False
         #: (:obj:`bool`) show high mask value  widget
         self.showhighvaluemask = False
         #: (:obj:`bool`) show statistics widget
@@ -67,6 +77,8 @@ class Settings(object):
         self.calcvariance = False
         #: (:obj:`bool`) show bakcground subtraction widget
         self.showsub = True
+        #: (:obj:`bool`) show bakcground subtraction scaling widget
+        self.showsubsf = False
         #: (:obj:`bool`) show transformation widget
         self.showtrans = True
         #: (:obj:`bool`) show memory buffer widget
@@ -87,6 +99,8 @@ class Settings(object):
         self.autodownsample = False
         #: (:obj:`bool`) keep original coordinates
         self.keepcoords = False
+        #: (:obj:`bool`) accelerate buffer sum
+        self.accelbuffersum = False
         #: (:obj:`bool`) lazy image slider
         self.lazyimageslider = True
         #: (:obj:`str`) security stream port
@@ -109,6 +123,8 @@ class Settings(object):
         self.zeromask = False
         #: (:obj:`bool`) nan mask enabled
         self.nanmask = True
+        #: (:obj:`bool`) adding maximal type value to negative high value mask
+        self.negmask = False
         #: (:obj:`bool`) security stream options
         self.secsockopt = b""
         #: (:obj:`float`) refresh rate is s
@@ -121,10 +137,14 @@ class Settings(object):
         self.interruptonerror = True
         #: (:obj:`str`) last image file name
         self.imagename = None
+        #: (:obj:`str`) calibration file name
+        self.calibrationfilename = None
         #: (:obj:`str`) last mask image file name
         self.maskimagename = None
-        #: (:obj:`str`) last background image file name
+        #: (:obj:`str`) last background/darkfield image file name
         self.bkgimagename = None
+        #: (:obj:`str`) last brightfield image file name
+        self.bfimagename = None
         #: (:obj:`bool`) statistics without scaling
         self.statswoscaling = True
         #: (:obj:`list` < :obj:`str` > ) zmq source topics
@@ -176,6 +196,18 @@ class Settings(object):
         self.nxslast = False
         #: (:obj:`list` < :obj:`str`>) hidra detector server list
         self.detservers = "[]"
+
+        #: (:obj:`str`) asapo server
+        self.asaposerver = ""
+        #: (:obj:`str`) asapo token
+        self.asapotoken = ""
+        #: (:obj:`str`) asapo beamtime id
+        self.asapobeamtime = ""
+        #: (:obj:`str`) asapo sourcepath
+        self.asaposourcepath = ""
+        #: (:obj:`list` < :obj:`str` > ) asapo datasources
+        self.asapodatasources = []
+
         #: (:obj:`bool`) use default detector servers
         self.defdetservers = True
         #: (:obj:`bool`) store detector geometry
@@ -198,6 +230,14 @@ class Settings(object):
         #: (:obj:`str`)  json list with tool widget names
         self.toolwidgets = "[]"
 
+        #: (:class:`pyFAI.azimuthalIntegrator.AzimuthalIntegrator`)
+        #       azimuthal integrator
+        self.ai = None
+        #: (:class:`pyFAI.azimuthalIntegrator.AzimuthalIntegrator`)
+        #       azimuthal integrator
+        #: (:class:`pyqtgraph.QtCore.QMutex`) ai mutex
+        self.aimutex = QtCore.QMutex()
+
         #: (:obj:`float`) x-coordinates of the center of the image
         self.centerx = 0.0
         #: (:obj:`float`) y-coordinates of the center of the image
@@ -210,10 +250,32 @@ class Settings(object):
         self.pixelsizey = 0.0
         #: (:obj:`float`) detector distance in mm
         self.detdistance = 0.0
+        #: (:obj:`float`) poni1 parameter in m
+        self.detponi1 = 0.0
+        #: (:obj:`float`) poni2 parameter in m
+        self.detponi2 = 0.0
+        #: (:obj:`float`) rot1 parameter in rad
+        self.detrot1 = 0.0
+        #: (:obj:`float`) rot2 parameter in rad
+        self.detrot2 = 0.0
+        #: (:obj:`float`) rot3 parameter in rad
+        self.detrot3 = 0.0
+        #: (:obj:`str`) detector name
+        self.detname = ""
+        #: (:obj:`str`) detector splineFile
+        self.detsplinefile = ""
+        #: (:obj:`int`) number of points for diffractogram
+        self.diffnpt = 1000
+        #: (:obj:`bool`) correct solid angle flag
+        self.correctsolidangle = True
         #: (:obj:`bool`) show all rois flag
         self.showallrois = False
+        #: (:obj:`bool`) single rois flag
+        self.singlerois = False
         #: (:obj:`bool`) send rois to LavueController flag
         self.sendrois = False
+        #: (:obj:`bool`) send results to LavueController flag
+        self.sendresults = False
         #: (:obj:`dict` < :obj:`str`, :obj:`dict` < :obj:`str`,`any`> >
         #                custom gradients
         self.__customgradients = {}
@@ -225,8 +287,12 @@ class Settings(object):
         self.__sourcedisplay = {}
 
         self.__units = {
+            'eV': ['eV'],
+            'keV': ['keV'],
+            'MeV': ['MeV'],
             'A': ['A', 'Angstrom'],
             'm': ['m', 'meter', 'metre'],
+            'nm': ['nm', 'nanometer', 'nanometre'],
             'um': ['um', 'micrometer', 'micrometre'],
             'mm': ['mm', 'millimeter', 'millimetre'],
             'cm': ['cm', 'centimeter', 'centimetre'],
@@ -234,7 +300,7 @@ class Settings(object):
             'pixel': ['pixels', 'pixel'],
         }
 
-        #: (:class:`pyqtgraph.QtCore.QMutex`) zmq bind address
+        #: (:class:`pyqtgraph.QtCore.QMutex`) settings mutex
         self.__mutex = QtCore.QMutex()
 
     def setCustomGradients(self, gradients):
@@ -282,6 +348,12 @@ class Settings(object):
         with QtCore.QMutexLocker(self.__mutex):
             if source in self.__sourcedisplay.keys():
                 values = dict(self.__sourcedisplay[str(source)])
+                if 'maskfile' not in values.keys():
+                    values['maskfile'] = None
+                if 'bkgfile' not in values.keys():
+                    values['bkgfile'] = None
+                if 'levels' not in values.keys():
+                    values['autofactor'] = True
         return values
 
     def load(self, settings):
@@ -304,6 +376,9 @@ class Settings(object):
         qstval = str(settings.value("Configuration/OrderROIs", type=str))
         if qstval.lower() == "true":
             self.orderrois = True
+        qstval = str(settings.value("Configuration/ImageChannels", type=str))
+        if qstval.lower() == "true":
+            self.imagechannels = True
         qstval = str(settings.value("Configuration/SecAutoPort", type=str))
         if qstval.lower() == "false":
             self.secautoport = False
@@ -316,6 +391,10 @@ class Settings(object):
             "Configuration/ShowSubtraction", type=str))
         if qstval.lower() == "false":
             self.showsub = False
+        qstval = str(settings.value(
+            "Configuration/ShowSubtractionScaling", type=str))
+        if qstval.lower() == "true":
+            self.showsubsf = True
         qstval = str(settings.value(
             "Configuration/ShowTransformations", type=str))
         if qstval.lower() == "false":
@@ -381,6 +460,10 @@ class Settings(object):
             "Configuration/KeepOriginalCoordinates", type=str))
         if qstval.lower() == "true":
             self.keepcoords = True
+        qstval = str(settings.value(
+            "Configuration/AccelerateBufferSum", type=str))
+        if qstval.lower() == "true":
+            self.accelbuffersum = True
         qstval = str(settings.value(
             "Configuration/LazyImageSlider", type=str))
         if qstval.lower() == "false":
@@ -391,9 +474,16 @@ class Settings(object):
         qstval = str(settings.value("Configuration/NXSLastImage", type=str))
         if qstval.lower() == "true":
             self.nxslast = True
+        qstval = str(settings.value(
+            "Configuration/SingleROIAliases", type=str))
+        if qstval.lower() == "true":
+            self.singlerois = True
         qstval = str(settings.value("Configuration/SendROIs", type=str))
         if qstval.lower() == "true":
             self.sendrois = True
+        qstval = str(settings.value("Configuration/SendToolResults", type=str))
+        if qstval.lower() == "true":
+            self.sendresults = True
         qstval = str(settings.value("Configuration/ShowAllROIs", type=str))
         if qstval.lower() == "true":
             self.showallrois = True
@@ -436,6 +526,10 @@ class Settings(object):
             "Configuration/MaskingAsNAN", type=str))
         if qstval.lower() == "false":
             self.nanmask = False
+        qstval = str(settings.value(
+            "Configuration/AddMaxValueToNegativeMask", type=str))
+        if qstval.lower() == "true":
+            self.negmask = True
 
         qstval = str(settings.value("Configuration/SecStream", type=str))
         if qstval.lower() == "true":
@@ -445,6 +539,8 @@ class Settings(object):
                     self.secsocket.bind(self.secsockopt)
                     self.secport = unicode(self.secsocket.getsockopt(
                         zmq.LAST_ENDPOINT)).split(":")[-1]
+                    if self.secport.endswith("'"):
+                        self.secport = self.secport[:-1]
                 else:
                     self.secsockopt = b"tcp://*:%s" % self.secport
                     self.secsocket.bind(self.secsockopt)
@@ -493,6 +589,15 @@ class Settings(object):
                 "Configuration/LastBackgroundImageFileName", type=str))
         if qstval:
             self.bkgimagename = qstval
+        qstval = str(
+            settings.value(
+                "Configuration/LastBrightFieldImageFileName", type=str))
+        if qstval:
+            self.bfimagename = qstval
+        qstval = str(
+            settings.value("Configuration/LastCalibrationFileName", type=str))
+        if qstval:
+            self.calibrationfilename = qstval
 
         qstval = str(
             settings.value(
@@ -518,6 +623,51 @@ class Settings(object):
         if qstval:
             self.zmqtopics = [str(tp) for tp in qstval]
 
+        qstval2 = \
+            settings.value(
+                "Configuration/ASAPOStreams", type=str)
+        qstval = \
+            settings.value(
+                "Configuration/ASAPODataSources", type=str)
+        if qstval:
+            self.asapodatasources = [str(tp) for tp in qstval]
+        elif qstval2:
+            self.asapodatasources = [str(tp) for tp in qstval2]
+        else:
+            self.asapodatasources = []
+
+        qstval = \
+            settings.value(
+                "Configuration/ASAPOServer", type=str)
+        if qstval:
+            self.asaposerver = str(qstval)
+        else:
+            self.asaposerver = ""
+
+        qstval = \
+            settings.value(
+                "Configuration/ASAPOToken", type=str)
+        if qstval:
+            self.asapotoken = str(qstval)
+        else:
+            self.asapotoken = ""
+
+        qstval = \
+            settings.value(
+                "Configuration/ASAPOBeamtime", type=str)
+        if qstval:
+            self.asapobeamtime = str(qstval)
+        else:
+            self.asapobeamtime = ""
+
+        qstval = \
+            settings.value(
+                "Configuration/ASAPOSourcePath", type=str)
+        if qstval:
+            self.asaposourcepath = str(qstval)
+        else:
+            self.asaposourcepath = ""
+
         qstval = \
             settings.value(
                 "Configuration/HidraDetectorServers", type=str)
@@ -538,6 +688,7 @@ class Settings(object):
             "Configuration/AutoZMQStreamTopics", type=str))
         if qstval.lower() == "true":
             self.autozmqtopics = True
+
         qstval = str(
             settings.value("Configuration/DirectoryTranslation", type=str))
         if qstval:
@@ -654,10 +805,90 @@ class Settings(object):
                 settings.value("Tools/DetectorDistance", type=str))
         except Exception:
             pass
+        try:
+            self.detponi1 = float(
+                settings.value("Tools/DetectorPONI1", type=str))
+        except Exception:
+            pass
+        try:
+            self.detponi2 = float(
+                settings.value("Tools/DetectorPONI2", type=str))
+        except Exception:
+            pass
+        try:
+            self.diffnpt = float(
+                settings.value("Tools/DiffractogramNPT", type=str))
+        except Exception:
+            pass
+        qstval = str(settings.value(
+            "Tools/CorrectSolidAngle", type=str))
+        if qstval.lower() == "false":
+            self.correctsolidangle = False
+        try:
+            self.detrot1 = float(
+                settings.value("Tools/DetectorRot1", type=str))
+        except Exception:
+            pass
+        try:
+            self.detrot2 = float(
+                settings.value("Tools/DetectorRot2", type=str))
+        except Exception:
+            pass
+        try:
+            self.detrot3 = float(
+                settings.value("Tools/DetectorRot3", type=str))
+        except Exception:
+            pass
+        qstval = str(
+            settings.value("Tools/DetectorName", type=str))
+        if qstval:
+            self.detname = qstval
+        qstval = str(
+            settings.value("Tools/DetectorSplineFile",
+                           type=str))
+        if qstval:
+            self.detsplinefile = qstval
+
         self.__loadDisplayParams(settings)
         self.__loadCustomGradients(settings)
+        self.updateAISettings()
         return status
 
+    def updateAISettings(self):
+        """ update AI settings
+        """
+        if PYFAI:
+            wvln = self.energy2m(self.energy)
+            pixel1 = self.distance2m((self.pixelsizey, "um"))
+            pixel2 = self.distance2m((self.pixelsizex, "um"))
+            splineFile = self.detsplinefile or None
+            detector = self.detname or None
+            detdistance = self.distance2m((self.detdistance, "mm"))
+
+            with QtCore.QMutexLocker(self.aimutex):
+                if wvln and detdistance and pixel1 and pixel2:
+                    from pyFAI.azimuthalIntegrator import AzimuthalIntegrator
+                    self.ai = AzimuthalIntegrator(
+                        dist=detdistance,
+                        poni1=self.detponi1,
+                        poni2=self.detponi2,
+                        rot1=self.detrot1,
+                        rot2=self.detrot2,
+                        rot3=self.detrot3,
+                        pixel1=pixel1,
+                        pixel2=pixel2,
+                        splineFile=splineFile,
+                        detector=detector,
+                        wavelength=wvln)
+                    if not self.detponi1 and not self.detponi2 \
+                       and self.centerx and self.centery:
+                        aif = self.ai.getFit2D()
+                        self.ai.setFit2D(aif["directDist"],
+                                         self.centerx,
+                                         self.centery,
+                                         aif["tilt"],
+                                         aif["tiltPlanRotation"])
+
     def store(self, settings):
         """ Stores settings in QSettings object
 
@@ -670,9 +901,15 @@ class Settings(object):
         settings.setValue(
             "Configuration/OrderROIs",
             self.orderrois)
+        settings.setValue(
+            "Configuration/ImageChannels",
+            self.imagechannels)
         settings.setValue(
             "Configuration/ShowSubtraction",
             self.showsub)
+        settings.setValue(
+            "Configuration/ShowSubtractionScaling",
+            self.showsubsf)
         settings.setValue(
             "Configuration/ShowTransformations",
             self.showtrans)
@@ -751,6 +988,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/MaskingAsNAN",
             self.nanmask)
+        settings.setValue(
+            "Configuration/AddMaxValueToNegativeMask",
+            self.negmask)
         settings.setValue(
             "Configuration/Sardana",
             self.sardana)
@@ -772,6 +1012,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/KeepOriginalCoordinates",
             self.keepcoords)
+        settings.setValue(
+            "Configuration/AccelerateBufferSum",
+            self.accelbuffersum)
         settings.setValue(
             "Configuration/LazyImageSlider",
             self.lazyimageslider)
@@ -781,9 +1024,15 @@ class Settings(object):
         settings.setValue(
             "Configuration/LastMaskImageFileName",
             self.maskimagename)
+        settings.setValue(
+            "Configuration/LastCalibrationFileName",
+            self.calibrationfilename)
         settings.setValue(
             "Configuration/LastBackgroundImageFileName",
             self.bkgimagename)
+        settings.setValue(
+            "Configuration/LastBrightFieldImageFileName",
+            self.bfimagename)
         settings.setValue(
             "Configuration/StatisticsWithoutScaling",
             self.statswoscaling)
@@ -796,6 +1045,21 @@ class Settings(object):
         settings.setValue(
             "Configuration/ZMQStreamTopics",
             self.zmqtopics)
+        settings.setValue(
+            "Configuration/ASAPODataSources",
+            self.asapodatasources)
+        settings.setValue(
+            "Configuration/ASAPOServer",
+            self.asaposerver)
+        settings.setValue(
+            "Configuration/ASAPOToken",
+            self.asapotoken)
+        settings.setValue(
+            "Configuration/ASAPOBeamtime",
+            self.asapobeamtime)
+        settings.setValue(
+            "Configuration/ASAPOSourcePath",
+            self.asaposourcepath)
         settings.setValue(
             "Configuration/HidraDetectorServers",
             self.detservers)
@@ -850,6 +1114,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/StoreGeometry",
             self.storegeometry)
+        settings.setValue(
+            "Tools/CorrectSolidAngle",
+            self.correctsolidangle)
         settings.setValue(
             "Configuration/GeometryFromSource",
             self.geometryfromsource)
@@ -862,6 +1129,12 @@ class Settings(object):
         settings.setValue(
             "Configuration/SendROIs",
             self.sendrois)
+        settings.setValue(
+            "Configuration/SendToolResults",
+            self.sendresults)
+        settings.setValue(
+            "Configuration/SingleROIAliases",
+            self.singlerois)
         settings.setValue(
             "Configuration/ShowAllROIs",
             self.showallrois)
@@ -876,6 +1149,11 @@ class Settings(object):
             self.pixelsizex = 0.0
             self.pixelsizey = 0.0
             self.detdistance = 0.0
+            self.detrot1 = 0.0
+            self.detrot2 = 0.0
+            self.detrot3 = 0.0
+            self.detponi1 = 0.0
+            self.detponi2 = 0.0
 
         settings.setValue(
             "Tools/CenterX",
@@ -895,6 +1173,31 @@ class Settings(object):
         settings.setValue(
             "Tools/DetectorDistance",
             self.detdistance)
+        settings.setValue(
+            "Tools/DetectorPONI1",
+            self.detponi1)
+        settings.setValue(
+            "Tools/DetectorPONI2",
+            self.detponi2)
+        settings.setValue(
+            "Tools/DiffractogramNPT",
+            self.diffnpt)
+        settings.setValue(
+            "Tools/DetectorRot1",
+            self.detrot1)
+        settings.setValue(
+            "Tools/DetectorRot2",
+            self.detrot2)
+        settings.setValue(
+            "Tools/DetectorRot3",
+            self.detrot3)
+        settings.setValue(
+            "Tools/DetectorName",
+            self.detname)
+        settings.setValue(
+            "Tools/DetectorSplineFile",
+            self.detsplinefile)
+
         self.__storeDisplayParams(settings)
         self.__storeCustomGradients(settings)
 
@@ -912,22 +1215,62 @@ class Settings(object):
         if length is not None:
             if type(length) in [list, tuple]:
                 if len(length) == 1:
-                    energy = hc * 10000 / length[0]
+                    if length[0]:
+                        energy = hc * 10000 / length[0]
                 elif len(length) > 1:
                     if length[1] in self.__units['A']:
-                        energy = hc * 10000 / length[0]
+                        if length[0]:
+                            energy = hc * 10000 / length[0]
+                    elif length[1] in self.__units['nm']:
+                        if length[0]:
+                            energy = hc * 1000 / length[0]
                     elif length[1] in self.__units['um']:
-                        energy = hc / length[0]
+                        if length[0]:
+                            energy = hc / length[0]
                     elif length[1] in self.__units['m']:
-                        energy = hc * 1e-6 / length[0]
+                        if length[0]:
+                            energy = hc * 1e-6 / length[0]
                     elif length[1] in self.__units['mm']:
-                        energy = hc * 1e-3 / length[0]
+                        if length[0]:
+                            energy = hc * 1e-3 / length[0]
                     elif length[1] in self.__units['cm']:
-                        energy = hc * 1e-4 / length[0]
+                        if length[0]:
+                            energy = hc * 1e-4 / length[0]
             else:
-                energy = hc * 10000 / length
+                if length:
+                    energy = hc * 10000 / length
         return energy
 
+    def energy2m(self, energy):
+        """ converts energy to m
+
+        :param energy: energy as value in eV or tuple with units
+        :type energy: :obj:`float` or (:obj:`float`, :obj:`str`)
+        :returns: length in m
+        :rtype: :obj:`float`
+        """
+        length = None
+        #: :obj:`float`  value of hc in eV * um
+        hc = 1.23984193
+        if energy is not None:
+            if type(energy) in [list, tuple]:
+                if len(energy) == 1:
+                    if energy[0]:
+                        length = hc * 1e-6 / energy[0]
+                elif len(energy) > 1:
+                    if energy[1] in self.__units['eV']:
+                        if energy[0]:
+                            length = hc * 1e-6 / energy[0]
+                    elif energy[1] in self.__units['keV']:
+                        if energy[0]:
+                            length = hc * 1e-3 / energy[0]
+                    elif energy[1] in self.__units['MeV']:
+                        if energy[0]:
+                            length = hc / energy[0]
+            elif energy:
+                length = hc * 1e-6 / energy
+        return length
+
     def distance2mm(self, distance):
         """ converts distance to mm units
 
@@ -944,6 +1287,8 @@ class Settings(object):
                 elif len(distance) > 1:
                     if distance[1] in self.__units['A']:
                         res = distance[0] * 1e-7
+                    elif distance[1] in self.__units['nm']:
+                        res = distance[0] * 1e-6
                     elif distance[1] in self.__units['um']:
                         res = distance[0] * 1e-3
                     elif distance[1] in self.__units['km']:
@@ -953,7 +1298,39 @@ class Settings(object):
                     elif distance[1] in self.__units['mm']:
                         res = distance[0]
                     elif distance[1] in self.__units['cm']:
-                        res = distance * 1e-1
+                        res = distance * 1e+1
+            else:
+                res = distance * 1e+3
+        return res
+
+    def distance2m(self, distance):
+        """ converts distance to m units
+
+        :param distance: distance as value in m or tuple with units
+        :type distance: :obj:`float` or (:obj:`float`, :obj:`str`)
+        :returns: distance in m
+        :rtype: :obj:`float`
+        """
+        res = None
+        if distance is not None:
+            if type(distance) in [list, tuple]:
+                if len(distance) == 1:
+                    res = distance[0] * 1e+3
+                elif len(distance) > 1:
+                    if distance[1] in self.__units['A']:
+                        res = distance[0] * 1e-10
+                    elif distance[1] in self.__units['nm']:
+                        res = distance[0] * 1e-9
+                    elif distance[1] in self.__units['um']:
+                        res = distance[0] * 1e-6
+                    elif distance[1] in self.__units['km']:
+                        res = distance[0] * 1e+3
+                    elif distance[1] in self.__units['m']:
+                        res = distance[0]
+                    elif distance[1] in self.__units['mm']:
+                        res = distance[0] * 1e-3
+                    elif distance[1] in self.__units['cm']:
+                        res = distance * 1e-2
             else:
                 res = distance * 1e+3
         return res
@@ -974,6 +1351,8 @@ class Settings(object):
                 elif len(distance) > 1:
                     if distance[1] in self.__units['A']:
                         res = distance[0] * 1e-4
+                    elif distance[1] in self.__units['nm']:
+                        res = distance[0] * 1e-3
                     elif distance[1] in self.__units['um']:
                         res = distance[0]
                     elif distance[1] in self.__units['km']:
@@ -1006,6 +1385,8 @@ class Settings(object):
                 elif len(distance) > 1:
                     if distance[1] in self.__units['pixel']:
                         res = distance[0]
+                    elif distance[1] in self.__units['nm']:
+                        res = distance[0] * 1e+3 / psize
                     elif distance[1] in self.__units['um']:
                         res = distance[0] / psize
                     elif distance[1] in self.__units['m']:
@@ -1085,6 +1466,33 @@ class Settings(object):
                     resy = self.distance2pixels([py, uy], self.pixelsizey)
         return resx, resy
 
+    def updateDetectorParameters(self):
+        """ update physical parameters
+
+        """
+        if PYFAI:
+            with QtCore.QMutexLocker(self.aimutex):
+                aic = self.ai.get_config()
+                self.pixelsizex = self.distance2um(
+                    (self.ai.get_pixel2(), "m"))
+                self.pixelsizey = self.distance2um(
+                    (self.ai.get_pixel1(), "m"))
+            self.detponi1 = aic["poni1"]
+            self.detponi2 = aic["poni2"]
+            self.detrot1 = aic["rot1"]
+            self.detrot2 = aic["rot2"]
+            self.detrot3 = aic["rot3"]
+            self.detsplinefile = self.ai.splineFile
+            self.detname = aic["detector"]
+            self.detdistance = self.distance2mm(
+                (aic["dist"], "m"))
+            self.energy = self.length2ev(
+                (float(aic["wavelength"]), "m"))
+            with QtCore.QMutexLocker(self.aimutex):
+                aif = self.ai.getFit2D()
+                self.centerx = float(aif["centerX"])
+                self.centery = float(aif["centerY"])
+
     def updateMetaData(self, **kargs):
         """ update physical parameters
 
@@ -1142,20 +1550,20 @@ class Settings(object):
                 self.centerx = cx
                 self.centery = cy
         if "beam_x" in kargs.keys():
-            cx = self.distance2pixels(kargs["beam_x"])
+            cx = self.distance2pixels(kargs["beam_x"], self.pixelsizex)
             if cx is not None:
                 self.centerx = cx
         if "beam_y" in kargs.keys():
-            cy = self.distance2pixels(kargs["beam_y"])
+            cy = self.distance2pixels(kargs["beam_y"], self.pixelsizey)
             if cy is not None:
                 self.centery = cy
 
         if "beam_center_x" in kargs.keys():
-            cx = self.distance2pixels(kargs["beam_center_x"])
+            cx = self.distance2pixels(kargs["beam_center_x"], self.pixelsizex)
             if cx is not None:
                 self.centerx = cx
         if "beam_center_y" in kargs.keys():
-            cy = self.distance2pixels(kargs["beam_center_y"])
+            cy = self.distance2pixels(kargs["beam_center_y"], self.pixelsizey)
             if cy is not None:
                 self.centery = cy
 
