@@ -46,8 +46,6 @@ class SourceForm(QtGui.QWidget):
 
     """ source form """
 
-    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) source connected signal
-    sourceConnected = QtCore.pyqtSignal()
     #: (:class:`pyqtgraph.QtCore.pyqtSignal`) source state signal
     sourceStateChanged = QtCore.pyqtSignal(int, int)
     #: (:class:`pyqtgraph.QtCore.pyqtSignal`) source state signal
@@ -359,11 +357,10 @@ class SourceForm(QtGui.QWidget):
                 self._emitAddIconClicked)
             self.__currentSource.removeIconClicked.disconnect(
                 self._emitRemoveIconClicked)
-            self.__currentSource.active = False
+            self.__currentSource.setActive(False)
             self.__currentSource.disconnectWidget()
         if name is not None and name in self.__sourcewidgets.keys():
             self.__currentSource = self.__sourcewidgets[name]
-            self.__currentSource.active = True
             self.__currentSource.buttonEnabled.connect(
                 self.emitButtonEnabled)
             self.__currentSource.sourceLabelChanged.connect(
@@ -374,6 +371,7 @@ class SourceForm(QtGui.QWidget):
                 self._emitAddIconClicked)
             self.__currentSource.removeIconClicked.connect(
                 self._emitRemoveIconClicked)
+            self.__currentSource.setActive(True)
         self.updateLayout()
         self.updateMetaData(disconnect=disconnect)
         self.emitSourceChanged()
@@ -454,10 +452,7 @@ class SourceForm(QtGui.QWidget):
         :param status: source state
         :type status: :obj:`int`
         """
-        if status == -1:
-            self.sourceConnected.emit()
-        else:
-            self.sourceStateChanged.emit(status, self.__sourceid)
+        self.sourceStateChanged.emit(status, self.__sourceid)
 
     @debugmethod
     @QtCore.pyqtSlot(bool)
@@ -775,6 +770,9 @@ class SourceTabWidget(QtGui.QTabWidget):
         if 2 not in self.__tabcheckboxstates:
             for sid in range(len(self.__tabcheckboxstates)):
                 self.setCheckState(sid, 2)
+        if self.isConnected():
+            self.toggleServerConnection()
+            self.toggleServerConnection()
 
     @debugmethod
     def tabCheckBoxStates(self):
